import datetime as dt
import json

from panther_core import PantherEvent
from panther_detection_helpers.caching import get_string_set, put_string_set

from pypanther import LogType, Rule, RuleMock, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_cloudtrail_success, aws_rule_context


@panther_managed
class AWSSSMDistributedCommand(Rule):
    id = "AWS.SSM.DistributedCommand-prototype"
    display_name = "AWS SSM Distributed Command"
    log_types = [LogType.AWS_CLOUDTRAIL]
    default_severity = Severity.INFO
    reports = {"MITRE ATT&CK": ["TA0002:T1203"]}
    default_description = "Detect an attacker utilizing AWS Systems Manager (SSM) to execute commands through SendCommand on multiple EC2 instances.\n"
    default_reference = "https://stratus-red-team.cloud/attack-techniques/AWS/aws.execution.ssm-send-command/\n"
    default_runbook = "Detetmine who issued the command, the command content and arguments, and which EC2 instances were affected. Determine the risk of an attacker creating a persistent point of access within one of the instances. Review behaviour logs for the EC2 instances (and their associated IAM roles).\n"
    summary_attributes = [
        "p_any_aws_account_ids",
        "p_any_aws_arns",
        "p_any_aws_instance_ids",
        "p_any_ip_addresses",
        "p_any_usernames",
    ]
    tags = ["AWS CloudTrail", "AWS SSM", "AWS EC2", "Execution: Exploitation for Client Execution", "Beta"]
    # Determine how separate instances need be commanded in order to trigger an alert
    INSTANCE_THRESHOLD = 2
    all_instance_ids = set()

    def rule(self, event: PantherEvent) -> bool:
        # Exclude events of the wrong type
        if event.get("eventName") != "SendCommand":
            return False
        # Determine if this actor accessed any other params in this account
        key = self.get_cache_key(event)
        cached_ids = self.get_cached_instance_ids(key)
        target_instance_ids = set(event.deep_get("requestParameters", "instanceIds", default=[]))
        # Determine if the cache needs updating with new entries
        self.all_instance_ids = cached_ids | target_instance_ids
        if self.all_instance_ids - cached_ids:
            # Only set the TTL if this is the first time we're adding to the cache
            #   Otherwise we'll be perpetually extending the lifespan of the cached data every time we
            #   add more.
            put_string_set(key, self.all_instance_ids, epoch_seconds=3600 if not cached_ids else None)
        # Check combined number of params
        return len(self.all_instance_ids) > self.INSTANCE_THRESHOLD

    def title(self, event: PantherEvent) -> str:
        actor = event.udm("actor_user")
        account_name = event.get("recipientAccountId")
        return f"Commands distributed to many EC2 instances by [{actor}] in [{account_name}]"

    def severity(self, event: PantherEvent) -> str:
        # Demote to LOW if attempt was denied
        if not aws_cloudtrail_success(event):
            return "LOW"
        return "DEFAULT"

    def alert_context(self, event: PantherEvent) -> dict:
        context = aws_rule_context(event)
        context.update({"instanceIds": list(self.all_instance_ids)})
        return context

    def get_cache_key(self, event) -> str:
        """
        Use the field values in the event to generate a cache key unique to this actor and
        account ID.
        """
        offset = dt.datetime.fromisoformat(event["p_event_time"]).timestamp() // 3600 * 3600
        actor = event.udm("actor_user")
        account = event.get("recipientAccountId")
        rule_id = "AWS.SSM.DistributedCommand"
        return f"{rule_id}-{account}-{actor}-{offset}"

    def get_cached_instance_ids(self, key: str) -> set[str]:
        """
        Get any previously cached parameter names. Included automatic converstion from string in
        the case of a unit test mock.
        """
        cached_ids = get_string_set(key, force_ttl_check=True)
        if isinstance(cached_ids, str):
            # This is a unit test
            cached_ids = set(json.loads(cached_ids))
        return cached_ids

    tests = [
        RuleTest(
            name="Send Command to Many Instances at Once",
            expected_result=True,
            mocks=[
                RuleMock(object_name="get_string_set", return_value="[]"),
                RuleMock(object_name="put_string_set", return_value=""),
            ],
            log={
                "p_event_time": "2025-02-19 16:32:39.000000000",
                "p_log_type": "AWS.CloudTrail",
                "p_parse_time": "2025-02-19 16:35:54.509896629",
                "awsRegion": "us-west-2",
                "eventCategory": "Management",
                "eventID": "ceaea85a-6db9-4595-9842-d904fce2f047",
                "eventName": "SendCommand",
                "eventSource": "ssm.amazonaws.com",
                "eventTime": "2025-02-19 16:32:39.000000000",
                "eventType": "AwsApiCall",
                "eventVersion": "1.08",
                "managementEvent": True,
                "readOnly": False,
                "recipientAccountId": "111122223333",
                "requestID": "c41499e3-c04c-48e7-9da9-ef63d0868553",
                "requestParameters": {
                    "documentName": "AWS-RunShellScript",
                    "instanceIds": ["i-006e4c07b5fba8ad2", "i-01bf673280e708b0d", "i-020001cca1a2f2628"],
                    "interactive": False,
                    "parameters": "HIDDEN_DUE_TO_SECURITY_REASONS",
                },
                "responseElements": {
                    "command": {
                        "alarmConfiguration": {"alarms": [], "ignorePollAlarmFailure": False},
                        "clientName": "",
                        "clientSourceId": "",
                        "cloudWatchOutputConfig": {"cloudWatchLogGroupName": "", "cloudWatchOutputEnabled": False},
                        "commandId": "f49a1fe5-d12b-4ac0-98bc-0e4bd83d70c0",
                        "comment": "",
                        "completedCount": 0,
                        "deliveryTimedOutCount": 0,
                        "documentName": "AWS-RunShellScript",
                        "documentVersion": "$DEFAULT",
                        "errorCount": 0,
                        "expiresAfter": "Feb 19, 2025, 6:32:39 PM",
                        "hasCancelCommandSignature": False,
                        "hasSendCommandSignature": False,
                        "instanceIds": ["i-006e4c07b5fba8ad2", "i-01bf673280e708b0d", "i-020001cca1a2f2628"],
                        "interactive": False,
                        "maxConcurrency": "50",
                        "maxErrors": "0",
                        "notificationConfig": {"notificationArn": "", "notificationEvents": [], "notificationType": ""},
                        "outputS3BucketName": "",
                        "outputS3KeyPrefix": "",
                        "outputS3Region": "us-west-2",
                        "parameters": "HIDDEN_DUE_TO_SECURITY_REASONS",
                        "requestedDateTime": "Feb 19, 2025, 4:32:39 PM",
                        "serviceRole": "",
                        "status": "Pending",
                        "statusDetails": "Pending",
                        "targetCount": 3,
                        "targets": [],
                        "timeoutSeconds": 3600,
                        "triggeredAlarms": [],
                    },
                },
                "sourceIPAddress": "1.2.3.4",
                "tlsDetails": {
                    "cipherSuite": "ECDHE-RSA-AES128-GCM-SHA256",
                    "clientProvidedHostHeader": "ssm.us-west-2.amazonaws.com",
                    "tlsVersion": "TLSv1.2",
                },
                "userAgent": "sample-user-agent",
                "userIdentity": {
                    "accessKeyId": "SAMPLE_ACCESS_KEY_ID",
                    "accountId": "111122223333",
                    "arn": "arn:aws:sts::111122223333:assumed-role/SampleRole/bobson.dugnutt",
                    "principalId": "SAMPLE_PRINCIPAL_ID:bobson.dugnutt",
                    "sessionContext": {
                        "attributes": {"creationDate": "2025-02-19T16:29:24Z", "mfaAuthenticated": "false"},
                        "sessionIssuer": {
                            "accountId": "111122223333",
                            "arn": "arn:aws:iam::111122223333:role/aws-reserved/sso.amazonaws.com/us-west-2/SampleRole",
                            "principalId": "SAMPLE_PRINCIPAL_ID",
                            "type": "Role",
                            "userName": "SampleRole",
                        },
                        "webIdFederationData": {},
                    },
                    "type": "AssumedRole",
                },
            },
        ),
        RuleTest(
            name="Single Command Send, Many Cached IDs",
            expected_result=True,
            mocks=[
                RuleMock(object_name="get_string_set", return_value="[1, 2, 3, 4, 5, 6, 7, 8, 9, 10]"),
                RuleMock(object_name="put_string_set", return_value=""),
            ],
            log={
                "p_event_time": "2025-02-19 16:32:39.000000000",
                "p_log_type": "AWS.CloudTrail",
                "p_parse_time": "2025-02-19 16:35:54.509896629",
                "awsRegion": "us-west-2",
                "eventCategory": "Management",
                "eventID": "ceaea85a-6db9-4595-9842-d904fce2f047",
                "eventName": "SendCommand",
                "eventSource": "ssm.amazonaws.com",
                "eventTime": "2025-02-19 16:32:39.000000000",
                "eventType": "AwsApiCall",
                "eventVersion": "1.08",
                "managementEvent": True,
                "readOnly": False,
                "recipientAccountId": "111122223333",
                "requestID": "c41499e3-c04c-48e7-9da9-ef63d0868553",
                "requestParameters": {
                    "documentName": "AWS-RunShellScript",
                    "instanceIds": ["i-006e4c07b5fba8ad2"],
                    "interactive": False,
                    "parameters": "HIDDEN_DUE_TO_SECURITY_REASONS",
                },
                "responseElements": {
                    "command": {
                        "alarmConfiguration": {"alarms": [], "ignorePollAlarmFailure": False},
                        "clientName": "",
                        "clientSourceId": "",
                        "cloudWatchOutputConfig": {"cloudWatchLogGroupName": "", "cloudWatchOutputEnabled": False},
                        "commandId": "f49a1fe5-d12b-4ac0-98bc-0e4bd83d70c0",
                        "comment": "",
                        "completedCount": 0,
                        "deliveryTimedOutCount": 0,
                        "documentName": "AWS-RunShellScript",
                        "documentVersion": "$DEFAULT",
                        "errorCount": 0,
                        "expiresAfter": "Feb 19, 2025, 6:32:39 PM",
                        "hasCancelCommandSignature": False,
                        "hasSendCommandSignature": False,
                        "instanceIds": ["i-006e4c07b5fba8ad2"],
                        "interactive": False,
                        "maxConcurrency": "50",
                        "maxErrors": "0",
                        "notificationConfig": {"notificationArn": "", "notificationEvents": [], "notificationType": ""},
                        "outputS3BucketName": "",
                        "outputS3KeyPrefix": "",
                        "outputS3Region": "us-west-2",
                        "parameters": "HIDDEN_DUE_TO_SECURITY_REASONS",
                        "requestedDateTime": "Feb 19, 2025, 4:32:39 PM",
                        "serviceRole": "",
                        "status": "Pending",
                        "statusDetails": "Pending",
                        "targetCount": 3,
                        "targets": [],
                        "timeoutSeconds": 3600,
                        "triggeredAlarms": [],
                    },
                },
                "sourceIPAddress": "1.2.3.4",
                "tlsDetails": {
                    "cipherSuite": "ECDHE-RSA-AES128-GCM-SHA256",
                    "clientProvidedHostHeader": "ssm.us-west-2.amazonaws.com",
                    "tlsVersion": "TLSv1.2",
                },
                "userAgent": "sample-user-agent",
                "userIdentity": {
                    "accessKeyId": "SAMPLE_ACCESS_KEY_ID",
                    "accountId": "111122223333",
                    "arn": "arn:aws:sts::111122223333:assumed-role/SampleRole/bobson.dugnutt",
                    "principalId": "SAMPLE_PRINCIPAL_ID:bobson.dugnutt",
                    "sessionContext": {
                        "attributes": {"creationDate": "2025-02-19T16:29:24Z", "mfaAuthenticated": "false"},
                        "sessionIssuer": {
                            "accountId": "111122223333",
                            "arn": "arn:aws:iam::111122223333:role/aws-reserved/sso.amazonaws.com/us-west-2/SampleRole",
                            "principalId": "SAMPLE_PRINCIPAL_ID",
                            "type": "Role",
                            "userName": "SampleRole",
                        },
                        "webIdFederationData": {},
                    },
                    "type": "AssumedRole",
                },
            },
        ),
        RuleTest(
            name="Single Command Send, No Cached IDs",
            expected_result=False,
            mocks=[
                RuleMock(object_name="get_string_set", return_value="[]"),
                RuleMock(object_name="put_string_set", return_value=""),
            ],
            log={
                "p_event_time": "2025-02-19 16:32:39.000000000",
                "p_log_type": "AWS.CloudTrail",
                "p_parse_time": "2025-02-19 16:35:54.509896629",
                "awsRegion": "us-west-2",
                "eventCategory": "Management",
                "eventID": "ceaea85a-6db9-4595-9842-d904fce2f047",
                "eventName": "SendCommand",
                "eventSource": "ssm.amazonaws.com",
                "eventTime": "2025-02-19 16:32:39.000000000",
                "eventType": "AwsApiCall",
                "eventVersion": "1.08",
                "managementEvent": True,
                "readOnly": False,
                "recipientAccountId": "111122223333",
                "requestID": "c41499e3-c04c-48e7-9da9-ef63d0868553",
                "requestParameters": {
                    "documentName": "AWS-RunShellScript",
                    "instanceIds": ["i-006e4c07b5fba8ad2"],
                    "interactive": False,
                    "parameters": "HIDDEN_DUE_TO_SECURITY_REASONS",
                },
                "responseElements": {
                    "command": {
                        "alarmConfiguration": {"alarms": [], "ignorePollAlarmFailure": False},
                        "clientName": "",
                        "clientSourceId": "",
                        "cloudWatchOutputConfig": {"cloudWatchLogGroupName": "", "cloudWatchOutputEnabled": False},
                        "commandId": "f49a1fe5-d12b-4ac0-98bc-0e4bd83d70c0",
                        "comment": "",
                        "completedCount": 0,
                        "deliveryTimedOutCount": 0,
                        "documentName": "AWS-RunShellScript",
                        "documentVersion": "$DEFAULT",
                        "errorCount": 0,
                        "expiresAfter": "Feb 19, 2025, 6:32:39 PM",
                        "hasCancelCommandSignature": False,
                        "hasSendCommandSignature": False,
                        "instanceIds": ["i-006e4c07b5fba8ad2"],
                        "interactive": False,
                        "maxConcurrency": "50",
                        "maxErrors": "0",
                        "notificationConfig": {"notificationArn": "", "notificationEvents": [], "notificationType": ""},
                        "outputS3BucketName": "",
                        "outputS3KeyPrefix": "",
                        "outputS3Region": "us-west-2",
                        "parameters": "HIDDEN_DUE_TO_SECURITY_REASONS",
                        "requestedDateTime": "Feb 19, 2025, 4:32:39 PM",
                        "serviceRole": "",
                        "status": "Pending",
                        "statusDetails": "Pending",
                        "targetCount": 3,
                        "targets": [],
                        "timeoutSeconds": 3600,
                        "triggeredAlarms": [],
                    },
                },
                "sourceIPAddress": "1.2.3.4",
                "tlsDetails": {
                    "cipherSuite": "ECDHE-RSA-AES128-GCM-SHA256",
                    "clientProvidedHostHeader": "ssm.us-west-2.amazonaws.com",
                    "tlsVersion": "TLSv1.2",
                },
                "userAgent": "sample-user-agent",
                "userIdentity": {
                    "accessKeyId": "SAMPLE_ACCESS_KEY_ID",
                    "accountId": "111122223333",
                    "arn": "arn:aws:sts::111122223333:assumed-role/SampleRole/bobson.dugnutt",
                    "principalId": "SAMPLE_PRINCIPAL_ID:bobson.dugnutt",
                    "sessionContext": {
                        "attributes": {"creationDate": "2025-02-19T16:29:24Z", "mfaAuthenticated": "false"},
                        "sessionIssuer": {
                            "accountId": "111122223333",
                            "arn": "arn:aws:iam::111122223333:role/aws-reserved/sso.amazonaws.com/us-west-2/SampleRole",
                            "principalId": "SAMPLE_PRINCIPAL_ID",
                            "type": "Role",
                            "userName": "SampleRole",
                        },
                        "webIdFederationData": {},
                    },
                    "type": "AssumedRole",
                },
            },
        ),
        RuleTest(
            name="Unrelated Event",
            expected_result=False,
            log={
                "p_event_time": "2025-02-19 18:00:45.000000000",
                "p_log_type": "AWS.CloudTrail",
                "p_parse_time": "2025-02-19 18:05:54.395474849",
                "awsRegion": "us-west-2",
                "eventCategory": "Management",
                "eventID": "447414b9-8516-4470-9e3a-f55067602ff0",
                "eventName": "DescribeInstances",
                "eventSource": "ec2.amazonaws.com",
                "eventTime": "2025-02-19 18:00:45.000000000",
                "eventType": "AwsApiCall",
                "eventVersion": "1.10",
                "managementEvent": True,
                "readOnly": True,
                "recipientAccountId": "111122223333",
                "requestID": "c44720ce-1853-41c8-96b0-06dac0284963",
                "requestParameters": {
                    "filterSet": {},
                    "instancesSet": {"items": [{"instanceId": "i-09c2e8f99d245cc69"}]},
                },
                "sourceIPAddress": "eks.amazonaws.com",
                "userAgent": "eks.amazonaws.com",
                "userIdentity": {
                    "accountId": "111122223333",
                    "arn": "arn:aws:sts::111122223333:assumed-role/SampleRole/SessionId",
                    "invokedBy": "eks.amazonaws.com",
                    "principalId": "SAMPLE_PRINCIPAL_ID:SessionId",
                    "sessionContext": {
                        "attributes": {"creationDate": "2025-02-19T17:55:44Z", "mfaAuthenticated": "false"},
                        "sessionIssuer": {
                            "accountId": "111122223333",
                            "arn": "arn:aws:iam::111122223333:role/SampleRole",
                            "principalId": "SAMPLE_PRINCIPAL_ID",
                            "type": "Role",
                            "userName": "SampleRole",
                        },
                    },
                    "type": "AssumedRole",
                },
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.gcp import gcp_alert_context


@panther_managed
class GCPCloudRunSetIAMPolicy(Rule):
    log_types = [LogType.GCP_AUDIT_LOG]
    default_description = "Detects new roles granted to users to Cloud Run Services. This could potentially allow the user to perform actions within the project and its resources, which could pose a security risk."
    display_name = "GCP Cloud Run Set IAM Policy"
    id = "GCP.Cloud.Run.Set.IAM.Policy-prototype"
    default_reference = "https://cloud.google.com/run/docs/securing/managing-access"
    default_runbook = "Confirm this was authorized and necessary behavior"
    default_severity = Severity.HIGH

    def rule(self, event):
        if event.get("severity") == "ERROR":
            return False
        method_name = event.deep_get("protoPayload", "methodName", default="")
        if not method_name.endswith("Services.SetIamPolicy"):
            return False
        authorization_info = event.deep_walk("protoPayload", "authorizationInfo")
        if not authorization_info:
            return False
        for auth in authorization_info:
            if auth.get("permission") == "run.services.setIamPolicy" and auth.get("granted") is True:
                return True
        return False

    def title(self, event):
        actor = event.deep_get("protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        resource = event.deep_get("protoPayload", "resourceName", default="<RESOURCE_NOT_FOUND>")
        service_name = resource.split("/")[-1] if "/" in resource else resource
        # Extract roles from the response bindings - there could be multiple
        bindings = event.deep_get("protoPayload", "response", "bindings", default=[])
        # Handle multiple roles if present
        roles = []
        for binding in bindings:
            if binding.get("role"):
                roles.append(binding.get("role"))
        # Format roles for title
        if not roles:
            roles_str = "<NO_ROLES_FOUND>"
        elif len(roles) == 1:
            roles_str = roles[0]
        else:
            # If multiple roles, mention the count and list the first one
            roles_str = f"{len(roles)} roles including {roles[0]}"
        project_id = event.deep_get("resource", "labels", "project_id", default="<PROJECT_NOT_FOUND>")
        return f"[GCP]: [{actor}] modified IAM policy for Cloud Run service [{service_name}] with [{roles_str}] in project [{project_id}]"

    def alert_context(self, event):
        context = gcp_alert_context(event)
        # Extract the service name from the resource path for better context
        resource = event.deep_get("protoPayload", "resourceName", default="")
        if resource:
            context["service_name"] = resource.split("/")[-1] if "/" in resource else resource
        # Get bindings and role information
        bindings = event.deep_get("protoPayload", "response", "bindings", default=[])
        # Collect all roles and members
        all_roles = []
        all_members = []
        role_to_members = {}
        for binding in bindings:
            role = binding.get("role")
            members = binding.get("members", [])
            if role:
                all_roles.append(role)
            if members:
                all_members.extend(members)
                # Create mapping of role to members
                if role:
                    role_to_members[role] = members
        # Store all collected information in the context
        context["assigned_roles"] = all_roles
        context["members_granted"] = all_members
        context["role_to_members_mapping"] = role_to_members
        return context

    tests = [
        RuleTest(
            name="GCP Run IAM Policy Set",
            expected_result=True,
            log={
                "insertId": "l3jvzyd2s2s",
                "logName": "projects/some-project/logs/cloudaudit.googleapis.com%2Factivity",
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {
                        "principalEmail": "some.user@company.com",
                        "principalSubject": "user:some.user@company.com",
                    },
                    "authorizationInfo": [
                        {
                            "granted": True,
                            "permission": "run.services.setIamPolicy",
                            "resource": "projects/some-project/locations/us-west1/services/cloudrun-exfil",
                            "resourceAttributes": {},
                        },
                        {"granted": True, "permission": "run.services.setIamPolicy", "resourceAttributes": {}},
                    ],
                    "methodName": "google.cloud.run.v1.Services.SetIamPolicy",
                    "request": {
                        "@type": "type.googleapis.com/google.iam.v1.SetIamPolicyRequest",
                        "policy": {
                            "bindings": [{"members": ["user:some.user@company.com"], "role": "roles/run.invoker"}],
                        },
                        "resource": "projects/some-project/locations/us-west1/services/cloudrun-exfil",
                    },
                    "requestMetadata": {
                        "callerIP": "1.2.3.4",
                        "callerSuppliedUserAgent": "(gzip),gzip(gfe)",
                        "destinationAttributes": {},
                        "requestAttributes": {"auth": {}, "time": "2024-02-02T09:44:26.173186Z"},
                    },
                    "resourceLocation": {"currentLocations": ["us-west1"]},
                    "resourceName": "projects/some-project/locations/us-west1/services/cloudrun-exfil",
                    "response": {
                        "@type": "type.googleapis.com/google.iam.v1.Policy",
                        "bindings": [{"members": ["user:some.user@company.com"], "role": "roles/run.invoker"}],
                        "etag": "BwYQYvUoBxs=",
                    },
                    "serviceName": "run.googleapis.com",
                },
                "receiveTimestamp": "2024-02-02 09:44:26.653891982",
                "resource": {
                    "labels": {
                        "configuration_name": "",
                        "location": "us-west1",
                        "project_id": "some-project",
                        "revision_name": "",
                        "service_name": "",
                    },
                    "type": "cloud_run_revision",
                },
                "severity": "NOTICE",
                "timestamp": "2024-02-02 09:44:26.029835000",
            },
        ),
        RuleTest(
            name="GCP Run IAM Policy Not Set",
            expected_result=False,
            log={
                "insertId": "l3jvzyd2s2s",
                "logName": "projects/some-project/logs/cloudaudit.googleapis.com%2Factivity",
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {
                        "principalEmail": "some.user@company.com",
                        "principalSubject": "user:some.user@company.com",
                    },
                    "authorizationInfo": [
                        {
                            "granted": False,
                            "permission": "run.services.setIamPolicy",
                            "resource": "projects/some-project/locations/us-west1/services/cloudrun-exfil",
                            "resourceAttributes": {},
                        },
                        {"granted": False, "permission": "run.services.setIamPolicy", "resourceAttributes": {}},
                    ],
                    "methodName": "google.cloud.run.v1.Services.SetIamPolicy",
                    "request": {
                        "@type": "type.googleapis.com/google.iam.v1.SetIamPolicyRequest",
                        "policy": {
                            "bindings": [{"members": ["user:some.user@company.com"], "role": "roles/run.invoker"}],
                        },
                        "resource": "projects/some-project/locations/us-west1/services/cloudrun-exfil",
                    },
                    "requestMetadata": "...",
                    "resourceLocation": "...",
                    "resourceName": "projects/some-project/locations/us-west1/services/cloudrun-exfil",
                    "serviceName": "run.googleapis.com",
                },
                "receiveTimestamp": "2024-02-02 09:44:26.653891982",
                "resource": "...",
                "severity": "NOTICE",
                "timestamp": "2024-02-02 09:44:26.029835000",
            },
        ),
        RuleTest(
            name="No method provided",
            expected_result=False,
            log={
                "insertId": "l3jvzyd2s2s",
                "logName": "projects/some-project/logs/cloudaudit.googleapis.com%2Factivity",
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {
                        "principalEmail": "some.user@company.com",
                        "principalSubject": "user:some.user@company.com",
                    },
                    "authorizationInfo": [
                        {
                            "granted": False,
                            "permission": "run.services.setIamPolicy",
                            "resource": "projects/some-project/locations/us-west1/services/cloudrun-exfil",
                            "resourceAttributes": {},
                        },
                        {"granted": False, "permission": "run.services.setIamPolicy", "resourceAttributes": {}},
                    ],
                    "request": {
                        "@type": "type.googleapis.com/google.iam.v1.SetIamPolicyRequest",
                        "policy": {
                            "bindings": [{"members": ["user:some.user@company.com"], "role": "roles/run.invoker"}],
                        },
                        "resource": "projects/some-project/locations/us-west1/services/cloudrun-exfil",
                    },
                    "requestMetadata": "...",
                    "resourceLocation": "...",
                    "resourceName": "projects/some-project/locations/us-west1/services/cloudrun-exfil",
                    "serviceName": "run.googleapis.com",
                },
                "receiveTimestamp": "2024-02-02 09:44:26.653891982",
                "resource": "...",
                "severity": "NOTICE",
                "timestamp": "2024-02-02 09:44:26.029835000",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class AxoniusWebhookCreated(Rule):
    id = "Axonius.WebhookCreated-prototype"
    display_name = "Axonius Webhook Created"
    log_types = [LogType.AXONIUS_ACTIVITY]
    tags = ["Axonius"]
    default_severity = Severity.LOW
    default_description = "Detects when an Axonius Webhook is Created"
    default_runbook = "Review if the webhook is approved and appropriate"

    def rule(self, event):
        action = event.deep_get("event", "action", default="")
        category = event.deep_get("event", "category", default="")
        if action == "AuditAction.Put" and category == "AuditCategory.WebhookManagement":
            return True
        return False

    def title(self, event):
        username = event.deep_get("event", "user", default="")
        return f"[Axonius] API Key Reset for {username} Detected"

    def alert_context(self, event):
        username = event.deep_get("event", "user", default="")
        config_id = event.deep_get("event", "params", "config_id", default="")
        vendor_name = event.deep_get("event", "params", "vendor_name", default="")
        event_type = event.deep_get("event", "type", default="")
        context = {"username": username, "config_id": config_id, "vendor_name": vendor_name, "type": event_type}
        return context

    tests = [
        RuleTest(
            name="Webhook Created",
            expected_result=True,
            log={
                "event": {
                    "action": "AuditAction.Put",
                    "category": "AuditCategory.WebhookManagement",
                    "params": {"config_id": "66db3fc88a93f66a4da0f084", "vendor_name": "Okta"},
                    "type": "user",
                    "user": "admin@axonius.com",
                },
                "source": "axonius",
                "time": 1745944321.285849,
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class GoogleWorkspaceAdminCustomRole(Rule):
    default_description = "A Google Workspace administrator created a new custom administrator role."
    display_name = "Google Workspace Admin Custom Role"
    default_runbook = "Please review this activity with the administrator and ensure this behavior was authorized."
    default_reference = "https://support.google.com/a/answer/2406043?hl=en#:~:text=under%20the%20limit.-,Create%20a%20custom%20role,-Before%20you%20begin"
    default_severity = Severity.MEDIUM
    tags = ["admin", "administrator", "google workspace", "role"]
    log_types = [LogType.GSUITE_ACTIVITY_EVENT]
    id = "Google.Workspace.Admin.Custom.Role-prototype"
    summary_attributes = ["name", "type"]

    def rule(self, event):
        # Return True to match the log event and trigger an alert.
        # Create Alert if there is a custom role created under delegated admin settings
        return event.get("type", "") == "DELEGATED_ADMIN_SETTINGS" and event.get("name", "") == "CREATE_ROLE"

    def title(self, event):
        # (Optional) Return a string which will be shown as the alert title.
        # If no 'dedup' function is defined, the return value of this method
        # will act as deduplication string.
        return f"Google Workspace Administrator [{event.get('actor', {}).get('email', 'NO_EMAIL_FOUND')}] created a new admin role [{event.get('parameters', {}).get('ROLE_NAME', 'NO_ROLE_NAME_FOUND')}]."

    tests = [
        RuleTest(
            name="Delete Role",
            expected_result=False,
            log={
                "actor": {"callerType": "USER", "email": "example@example.io", "profileId": "123456"},
                "id": {
                    "applicationName": "admin",
                    "customerId": "D12345",
                    "time": "2022-12-11 02:57:48.693000000",
                    "uniqueQualifier": "-12456",
                },
                "ipAddress": "12.12.12.12",
                "kind": "admin#reports#activity",
                "name": "DELETE_ROLE",
                "parameters": {"ROLE_ID": "567890", "ROLE_NAME": "CustomAdminRoleName"},
                "type": "DELEGATED_ADMIN_SETTINGS",
            },
        ),
        RuleTest(
            name="New Custom Role Created",
            expected_result=True,
            log={
                "actor": {"callerType": "USER", "email": "example@example.io", "profileId": "123456"},
                "id": {
                    "applicationName": "admin",
                    "customerId": "D12345",
                    "time": "2022-12-11 02:57:48.693000000",
                    "uniqueQualifier": "-12456",
                },
                "ipAddress": "12.12.12.12",
                "kind": "admin#reports#activity",
                "name": "CREATE_ROLE",
                "parameters": {"ROLE_ID": "567890", "ROLE_NAME": "CustomAdminRoleName"},
                "type": "DELEGATED_ADMIN_SETTINGS",
            },
        ),
        RuleTest(
            name="ListObject Type",
            expected_result=False,
            log={
                "actor": {"email": "user@example.io", "profileId": "118111111111111111111"},
                "id": {
                    "applicationName": "drive",
                    "customerId": "D12345",
                    "time": "2022-12-20 17:27:47.080000000",
                    "uniqueQualifier": "-7312729053723258069",
                },
                "ipAddress": "12.12.12.12",
                "kind": "admin#reports#activity",
                "name": "rename",
                "parameters": {
                    "actor_is_collaborator_account": None,
                    "billable": True,
                    "doc_id": "1GGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGG",
                    "doc_title": "Document Title- Found Here",
                    "doc_type": "presentation",
                    "is_encrypted": None,
                    "new_value": ["Document Title- Found Here"],
                    "old_value": ["Document Title- Old"],
                    "owner": "user@example.io",
                    "owner_is_shared_drive": None,
                    "owner_is_team_drive": None,
                    "primary_event": True,
                    "visibility": "private",
                },
                "type": "access",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.github import github_alert_context


@panther_managed
class GithubRepoVulnerabilityDismissed(Rule):
    display_name = "GitHub Dependabot Vulnerability Dismissed"
    id = "Github.Repo.VulnerabilityDismissed-prototype"
    default_severity = Severity.HIGH
    default_description = "Creates an alert if a dependabot alert is dismissed without being fixed.\n"
    log_types = [LogType.GITHUB_AUDIT]

    def rule(self, event):
        if event.get("action") == "repository_vulnerability_alert.dismiss":
            return True
        return False

    def title(self, event):
        return f"GitHub Dependabot Vulnerability Dismissed by {event.get('actor')}: {event.get('repo')}"

    def alert_context(self, event):
        context = github_alert_context(event)
        alert_url = f"https://github.com/{event.get('repo')}/security/dependabot/{event.get('alert_number')}"
        return context | {"alert_url": alert_url}

    tests = [
        RuleTest(
            name="Not GitHub Dependabot Vulnerability Dismissed",
            expected_result=False,
            log={"action": "not_repository_vulnerability_alert.dismiss"},
        ),
        RuleTest(
            name="GitHub Dependabot Vulnerability Dismissed",
            expected_result=True,
            log={
                "_document_id": "Z7JUOWzi2wKeWsZcOhbS9w",
                "action": "repository_vulnerability_alert.dismiss",
                "active": True,
                "actor": "badger",
                "actor_id": "1234567",
                "actor_is_bot": False,
                "alert_number": 8,
                "at_sign_timestamp": "2024-04-09 06:17:55.186000000",
                "business": "acme",
                "business_id": "11244",
                "created_at": "2024-03-13 22:36:27.788000000",
                "external_identity_nameid": "badger@acme.com",
                "ghsa_id": "GHSA-1234-5678-9090",
                "operation_type": "modify",
                "org": "acme",
                "org_id": 42053323,
                "public_repo": True,
                "repo": "acme/repo",
                "repo_id": 6532371245,
                "user": "badger",
                "user_agent": "Mozilla/5.0 (Macintosh Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/123.0.0.0 Safari/537.36",
                "user_id": "1270063",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.crowdstrike_event_streams import cs_alert_context


@panther_managed
class CrowdstrikeAPIKeyCreated(Rule):
    id = "Crowdstrike.API.Key.Created-prototype"
    display_name = "Crowdstrike API Key Created"
    reports = {"MITRE ATT&CK": ["TA0006:T1098.001"]}
    log_types = [LogType.CROWDSTRIKE_EVENT_STREAMS]
    default_severity = Severity.MEDIUM
    default_description = "A user created an API Key in CrowdStrike"
    default_runbook = "Reach out to the user if needed to validate the activity."

    def rule(self, event):
        return all([event.deep_get("event", "OperationName") == "CreateAPIClient", event.deep_get("event", "Success")])

    def title(self, event):
        user = event.deep_get("event", "UserId")
        service = event.deep_get("event", "ServiceName")
        return f"{user} created a new API key in {service}"

    def alert_context(self, event):
        return cs_alert_context(event)

    tests = [
        RuleTest(
            name="API Key Created",
            expected_result=True,
            log={
                "event": {
                    "AuditKeyValues": [
                        {"Key": "scope(s)", "ValueString": "alerts:read,api-integrations:read"},
                        {"Key": "actor_user", "ValueString": "tester@panther.com"},
                        {"Key": "actor_user_uuid", "ValueString": "a11a1111-1a11-1a1a-1a11-a11a111a111a"},
                        {"Key": "actor_cid", "ValueString": "aaa111111111111111aaaaaa11a11a11"},
                        {"Key": "trace_id", "ValueString": "1a111111-a1a1-111a-11aa-a111111a1a1a"},
                        {"Key": "APIClientID", "ValueString": "aaa1a11aaa111a1a11a11aaaa1aa1a11"},
                        {"Key": "id", "ValueString": "aaa1a11aaa111a1a11a11aaaa1aa1a11"},
                        {"Key": "name", "ValueString": "key name"},
                    ],
                    "OperationName": "CreateAPIClient",
                    "ServiceName": "Crowdstrike API Client",
                    "Success": True,
                    "UTCTimestamp": "2024-07-08 14:01:54.000000000",
                    "UserId": "tester@panther.com",
                    "UserIp": "11.1.111.11",
                },
                "metadata": {
                    "customerIDString": "aaa111111111111111aaaaaa11a11a11",
                    "eventCreationTime": "2024-07-08 14:01:54.451000000",
                    "eventType": "AuthActivityAuditEvent",
                    "offset": 111111,
                    "version": "1.0",
                },
            },
        ),
    ]



from base64 import b64decode

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class IntuneCreateModifyClientApp(Rule):
    id = "Intune.CreateModifyClientApp-prototype"
    display_name = "Intune Create or Modify Client App"
    log_types = [LogType.MICROSOFT_INTUNE_AUDIT_LOGS]
    tags = ["InTune"]
    default_severity = Severity.MEDIUM
    reports = {"MITRE ATT&CK": ["TA0002:T1072", "TA0008:T1021.007", "TA0005:T1202"]}
    default_description = "Microsoft Intune allows administrators to deploy applications to devices as a means of remote management and configuration. This functionality can be abused by adversaries to deploy malicious executables to devices, thereby allowing adversaries to pivot from compromised accounts to endpoints. This detection identifies the creation of or changes to apps that are deployed to devices."
    default_runbook = "Review the actions taken to determine if they are legitimate changes by an administrator. Microsoft does not provide the IP address of the actor or hashes of the deployed applications. The Targets field will contain information on the deployed application to include names and parameters. Note that it is also possible for administrators to configure a PowerShell script to confirm an app is installed. This can also be abused to execute malicious code. If a script is configured, it will be present in base64 encoded form in the ModifiedProperties field."
    default_reference = "https://www.ibm.com/think/x-force/detecting-intune-lateral-movement"
    ACTOR = OPERATION = ""

    def rule(self, event):
        self.OPERATION = event.get("operationName", "")
        # Alert on creation or modification of mobile apps
        return self.OPERATION.lower() in ["create mobileapp", "patch mobileapp"]

    def title(self, event):
        self.ACTOR = event.get("identity", "")
        if self.OPERATION.lower().startswith("create"):
            return f"Intune: [{self.ACTOR or '<N/A>'}] created a new Intune mobile app."
        return f"Intune: [{self.ACTOR or '<N/A>'}] modified an Intune mobile app"

    def alert_context(self, event):
        context = {
            "Actor": event.get("identity", default="Unknown"),
            "Operation": event.get("operationName", default="Unknown"),
            "Deployed App(s)": event.deep_get("properties", "TargetDisplayNames", default="Unknown"),
        }
        # Intune allows administrators to write PowerShell scripts to verify an app is properly
        # installed. These scripts can be used for malicious purposes. If a new script is added,
        # it will be in a ModifiedProperty section for hte given app, and will be base64 encoded
        scripts = []
        targets = event.deep_get("properties", "Targets")
        for target in targets:
            for prop in target["ModifiedProperties"]:
                if "Collection.Rules.ScriptContent" in prop.get("Name") and prop.get("New"):
                    scripts.append(b64decode(prop.get("New")).decode("utf8"))
        if scripts:
            context["Validation Script(s)"] = scripts
        return context

    tests = [
        RuleTest(
            name="Add Mobile App",
            expected_result=True,
            log={
                "category": "AuditLogs",
                "correlationId": "217da327-3767-4813-8f5f-af03ef38562a",
                "identity": "testuser@test.com",
                "operationName": "Create MobileApp",
                "properties": {
                    "ActivityDate": "4/11/2025 6:21:02 PM",
                    "ActivityResultStatus": 1,
                    "ActivityType": 0,
                    "Actor": {
                        "ActorType": 1,
                        "Application": "5926fc8e-304e-4f59-8bed-58ca97cc39a4",
                        "ApplicationName": "Microsoft Intune portal extension",
                        "IsDelegatedAdmin": False,
                        "ObjectId": "3a4518e8-d287-47a1-967e-cfde068d12e9",
                        "PartnerTenantId": "00000000-0000-0000-0000-000000000000",
                        "UPN": "testuser@test.com",
                        "UserPermissions": ["*"],
                    },
                    "AdditionalDetails": "",
                    "AuditEventId": "ead98102-798e-474c-a3ae-2483eac30532",
                    "Category": 5,
                    "TargetDisplayNames": ["yubikey-manager-qt-1.2.6-win64.exe"],
                    "TargetObjectIds": ["22a27bc3-334d-400b-b428-a05911dc29c4"],
                    "Targets": [
                        {
                            "ModifiedProperties": [
                                {
                                    "Name": "InstallCommandLine",
                                    "New": 'yubikey-manager-qt-1.2.6-win64.exe /D "C:\\Program Files\\Yubico\\YubiKey Manager" /S',
                                },
                                {
                                    "Name": "UninstallCommandLine",
                                    "New": 'ykman-uninstall.exe /D "C:\\Program Files\\Yubico\\YubiKey Manager" /S',
                                },
                                {"Name": "ApplicableArchitectures", "New": "X64"},
                                {"Name": "MinimumSupportedOperatingSystem.V8_0", "New": "False"},
                                {"Name": "MinimumSupportedOperatingSystem.V8_1", "New": "False"},
                                {"Name": "MinimumSupportedOperatingSystem.V10_0", "New": "False"},
                                {"Name": "MinimumSupportedOperatingSystem.V10_1607", "New": "True"},
                                {"Name": "MinimumSupportedOperatingSystem.V10_1703", "New": "False"},
                                {"Name": "MinimumSupportedOperatingSystem.V10_1709", "New": "False"},
                                {"Name": "MinimumSupportedOperatingSystem.V10_1803", "New": "False"},
                                {"Name": "MinimumSupportedOperatingSystem.V10_1809", "New": "False"},
                                {"Name": "MinimumSupportedOperatingSystem.V10_1903", "New": "False"},
                                {"Name": "MinimumSupportedOperatingSystem.V10_1909", "New": "False"},
                                {"Name": "MinimumSupportedOperatingSystem.V10_2004", "New": "False"},
                                {"Name": "MinimumSupportedOperatingSystem.V10_2H20", "New": "False"},
                                {"Name": "MinimumSupportedOperatingSystem.V10_21H1", "New": "False"},
                                {"Name": "MinimumFreeDiskSpaceInMB", "New": "<null>"},
                                {"Name": "MinimumMemoryInMB", "New": "<null>"},
                                {"Name": "MinimumNumberOfProcessors", "New": "<null>"},
                                {"Name": "MinimumCpuSpeedInMHz", "New": "<null>"},
                                {"Name": "InstallExperience.RunAsAccount", "New": "System"},
                                {"Name": "InstallExperience.MaxRunTimeInMinutes", "New": "60"},
                                {"Name": "InstallExperience.DeviceRestartBehavior", "New": "Allow"},
                                {"Name": "SetupFilePath", "New": "yubikey-manager-qt-1.2.6-win64.exe"},
                                {"Name": "MinimumSupportedWindowsRelease", "New": "1607"},
                                {"Name": "DisplayVersion", "New": ""},
                                {"Name": "AllowAvailableUninstall", "New": "True"},
                                {"Name": "CommittedContentVersion", "New": "<null>"},
                                {"Name": "FileName", "New": "yubikey-manager-qt-1.2.6-win64.intunewin"},
                                {"Name": "Size", "New": "0"},
                                {"Name": "Id", "New": "22a27bc3-334d-400b-b428-a05911dc29c4"},
                                {"Name": "Description", "New": "yubikey-manager-qt-1.2.6-win64.exe"},
                                {"Name": "Publisher", "New": "YubiCo"},
                                {"Name": "CreatedDateTime", "New": "4/11/2025 6:21:02 PM"},
                                {"Name": "LastModifiedDateTime", "New": "4/11/2025 6:21:02 PM"},
                                {"Name": "IsFeatured", "New": "False"},
                                {"Name": "PrivacyInformationUrl", "New": ""},
                                {"Name": "InformationUrl", "New": "<null>"},
                                {"Name": "Owner", "New": ""},
                                {"Name": "Developer", "New": ""},
                                {"Name": "Notes", "New": ""},
                                {"Name": "UploadState", "New": "0"},
                                {"Name": "PublishingState", "New": "NotPublished"},
                                {"Name": "IsAssigned", "New": "False"},
                                {"Name": "DependentAppCount", "New": "0"},
                                {"Name": "SupersedingAppCount", "New": "0"},
                                {"Name": "SupersededAppCount", "New": "0"},
                                {"Name": "DeviceManagementAPIVersion", "New": "5025-01-29"},
                                {"Name": "$Collection.DetectionRules.Check32BitOn64System[0]", "New": "False"},
                                {"Name": "$Collection.DetectionRules.DetectionType[0]", "New": "Exists"},
                                {"Name": "$Collection.DetectionRules.DetectionValue[0]", "New": "<null>"},
                                {"Name": "$Collection.DetectionRules.FileOrFolderName[0]", "New": "ykman-gui.exe"},
                                {"Name": "$Collection.DetectionRules.Operator[0]", "New": "NotConfigured"},
                                {
                                    "Name": "$Collection.DetectionRules.Path[0]",
                                    "New": "C:\\Program Files\\Yubico\\YubiKey Manager",
                                },
                                {"Name": "$Collection.ReturnCodes.ReturnCode[0]", "New": "0"},
                                {"Name": "$Collection.ReturnCodes.ReturnCode[1]", "New": "1707"},
                                {"Name": "$Collection.ReturnCodes.ReturnCode[2]", "New": "3010"},
                                {"Name": "$Collection.ReturnCodes.ReturnCode[3]", "New": "1641"},
                                {"Name": "$Collection.ReturnCodes.ReturnCode[4]", "New": "1618"},
                                {"Name": "$Collection.ReturnCodes.Type[0]", "New": "Success"},
                                {"Name": "$Collection.ReturnCodes.Type[1]", "New": "Success"},
                                {"Name": "$Collection.ReturnCodes.Type[2]", "New": "SoftReboot"},
                                {"Name": "$Collection.ReturnCodes.Type[3]", "New": "HardReboot"},
                                {"Name": "$Collection.ReturnCodes.Type[4]", "New": "Retry"},
                                {"Name": "$Collection.Rules.Check32BitOn64System[0]", "New": "False"},
                                {"Name": "$Collection.Rules.ComparisonValue[0]", "New": "<null>"},
                                {"Name": "$Collection.Rules.FileOrFolderName[0]", "New": "ykman-gui.exe"},
                                {"Name": "$Collection.Rules.OperationType[0]", "New": "Exists"},
                                {"Name": "$Collection.Rules.Operator[0]", "New": "NotConfigured"},
                                {
                                    "Name": "$Collection.Rules.Path[0]",
                                    "New": "C:\\Program Files\\Yubico\\YubiKey Manager",
                                },
                                {"Name": "$Collection.Rules.RuleType[0]", "New": "Detection"},
                            ],
                            "Name": "yubikey-manager-qt-1.2.6-win64.exe",
                        },
                    ],
                },
                "resultDescription": "None",
                "resultType": "Success",
                "tenantId": "11111111-2222-3333-44444-555555555555",
                "time": "2025-04-11:21:02.3387000Z",
            },
        ),
        RuleTest(
            name="Mobile app edited to include PowerShell",
            expected_result=True,
            log={
                "category": "AuditLogs",
                "correlationId": "729bb97b-641a-4da9-b7f0-49cdfd33552b",
                "identity": "testuser@test.com",
                "operationName": "Patch MobileApp",
                "properties": {
                    "ActivityDate": "4/25/2025 12:44:00 AM",
                    "ActivityResultStatus": 1,
                    "ActivityType": 2,
                    "Actor": {
                        "ActorType": 1,
                        "Application": "5926fc8e-304e-4f59-8bed-58ca97cc39a4",
                        "ApplicationName": "Microsoft Intune portal extension",
                        "IsDelegatedAdmin": False,
                        "ObjectId": "5774e54b-f2ad-41f8-a2d5-4bc356342fa6",
                        "PartnerTenantId": "00000000-0000-0000-0000-000000000000",
                        "UPN": "testuser@test.com",
                        "UserPermissions": ["*"],
                    },
                    "AdditionalDetails": "",
                    "AuditEventId": "ef69082e-23d7-428a-9c74-0f2cacad2572",
                    "Category": 5,
                    "TargetDisplayNames": ["yubikey-manager-qt-1.2.6-win64.exe"],
                    "TargetObjectIds": ["22a27bc3-334d-400b-b428-a05911dc29c4"],
                    "Targets": [
                        {
                            "ModifiedProperties": [
                                {"Name": "DeviceManagementAPIVersion", "New": "5025-01-29"},
                                {"Name": "$Collection.DetectionRules.Check32BitOn64System[0]", "Old": "False"},
                                {"Name": "$Collection.DetectionRules.DetectionType[0]", "Old": "Exists"},
                                {"Name": "$Collection.DetectionRules.DetectionValue[0]", "Old": "<null>"},
                                {"Name": "$Collection.DetectionRules.FileOrFolderName[0]", "Old": "ykman-gui.exe"},
                                {"Name": "$Collection.DetectionRules.Operator[0]", "Old": "NotConfigured"},
                                {
                                    "Name": "$Collection.DetectionRules.Path[0]",
                                    "Old": "C:\\Program Files\\Yubico\\YubiKey Manager",
                                },
                                {"Name": "$Collection.Rules.Check32BitOn64System[0]", "Old": "False"},
                                {"Name": "$Collection.Rules.DisplayName[0]", "New": "<null>"},
                                {"Name": "$Collection.Rules.EnforceSignatureCheck[0]", "New": "False"},
                                {"Name": "$Collection.Rules.FileOrFolderName[0]", "Old": "ykman-gui.exe"},
                                {"Name": "$Collection.Rules.OperationType[0]", "New": "NotConfigured"},
                                {"Name": "$Collection.Rules.OperationType[1]", "Old": "Exists"},
                                {
                                    "Name": "$Collection.Rules.Path[0]",
                                    "Old": "C:\\Program Files\\Yubico\\YubiKey Manager",
                                },
                                {"Name": "$Collection.Rules.RunAs32Bit[0]", "New": "False"},
                                {"Name": "$Collection.Rules.ScriptContent[0]", "New": "V3JpdGUtT3V0cHV0ICJ0ZXN0Ig=="},
                            ],
                            "Name": "yubikey-manager-qt-1.2.6-win64.exe",
                        },
                    ],
                },
                "resultDescription": "None",
                "resultType": "Success",
                "tenantId": "11111111-2222-3333-4444-55555555555",
                "time": "2025-04-25T00:44:00.4679000Z",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.zscaler import zia_alert_context, zia_success


@panther_managed
class ZIATrustModification(Rule):
    id = "ZIA.Trust.Modification-prototype"
    default_description = "This rule detects when SAML authentication was enabled/disabled."
    display_name = "ZIA Trust Modification"
    default_runbook = (
        "Verify that this change was planned. If not, revert the change and ensure this doesn't happen again."
    )
    default_reference = "https://help.zscaler.com/zia/configuring-saml"
    default_severity = Severity.MEDIUM
    reports = {"MITRE ATT&CK": ["TA0004:T1484.002"]}
    log_types = [LogType.ZSCALER_ZIA_ADMIN_AUDIT_LOG]

    def rule(self, event):
        if not zia_success(event):
            return False
        action = event.deep_get("event", "action", default="ACTION_NOT_FOUND")
        category = event.deep_get("event", "category", default="CATEGORY_NOT_FOUND")
        saml_enabled_pre = event.deep_get("event", "preaction", "samlEnabled", default="<PRE_SAML_STATUS_NOT_FOUND>")
        saml_enabled_post = event.deep_get("event", "postaction", "samlEnabled", default="<POST_SAML_STATUS_NOT_FOUND>")
        if action == "UPDATE" and category == "ADMINISTRATOR_MANAGEMENT" and (saml_enabled_pre != saml_enabled_post):
            return True
        return False

    def title(self, event):
        return f"[Zscaler.ZIA]: SAML configuration was changed by admin with id [{event.deep_get('event', 'adminid', default='<ADMIN_ID_NOT_FOUND>')}]"

    def alert_context(self, event):
        return zia_alert_context(event)

    tests = [
        RuleTest(
            name="Administration > Administration Management > Enable SAML Authentication",
            expected_result=True,
            log={
                "event": {
                    "action": "UPDATE",
                    "adminid": "admin@16991311.zscalerbeta.net",
                    "auditlogtype": "ZIA",
                    "category": "ADMINISTRATOR_MANAGEMENT",
                    "clientip": "123.123.123.123",
                    "errorcode": "None",
                    "interface": "UI",
                    "postaction": {"certFilename": "abc.crt", "productId": 0, "samlEnabled": True},
                    "preaction": {"productId": 0, "samlEnabled": False},
                    "recordid": "332",
                    "resource": "None",
                    "result": "SUCCESS",
                    "subcategory": "ADMINISTRATOR_SAML",
                    "time": "2024-10-22 22:13:23.000000000",
                },
                "sourcetype": "zscalernss-audit",
            },
        ),
        RuleTest(
            name="Administration > Administration Management > Disable SAML Authentication",
            expected_result=True,
            log={
                "event": {
                    "action": "UPDATE",
                    "adminid": "admin@16991311.zscalerbeta.net",
                    "auditlogtype": "ZIA",
                    "category": "ADMINISTRATOR_MANAGEMENT",
                    "clientip": "123.123.123.123",
                    "errorcode": "None",
                    "interface": "UI",
                    "postaction": {"certFilename": "abc.crt", "productId": 0, "samlEnabled": False},
                    "preaction": {"productId": 0, "samlEnabled": True},
                    "recordid": "332",
                    "resource": "None",
                    "result": "SUCCESS",
                    "subcategory": "ADMINISTRATOR_SAML",
                    "time": "2024-10-22 22:13:23.000000000",
                },
                "sourcetype": "zscalernss-audit",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_rule_context


@panther_managed
class AWSSoftwareDiscovery(Rule):
    default_description = (
        "A user is obtaining a list of security software, configurations, defensive tools, and sensors that are in AWS."
    )
    display_name = "AWS Software Discovery"
    enabled = False
    default_reference = "https://attack.mitre.org/techniques/T1518/001/"
    tags = ["Configuration Required"]
    reports = {"MITRE ATT&CK": ["TA0007:T1518"]}
    default_severity = Severity.INFO
    create_alert = False
    dedup_period_minutes = 360
    log_types = [LogType.AWS_CLOUDTRAIL]
    id = "AWS.Software.Discovery-prototype"
    threshold = 50
    DISCOVERY_EVENTS = [
        "ListDocuments",
        "ListMembers",
        "DescribeProducts",
        "DescribeStandards",
        "DescribeStandardsControls",
        "DescribeInstanceInformation",
        "DescribeSecurityGroups",
        "DescribeSecurityGroupRules",
        "DescribeSecurityGroupReferences",
        "DescribeSubnets",
        "DescribeHub",
        "ListFirewalls",
        "ListRuleGroups",
        "ListFirewallPolicies",
        "DescribeFirewall",
        "DescribeFirewallPolicy",
        "DescribeLoggingConfiguration",
        "DescribeResourcePolicy",
        "DescribeRuleGroup",
    ]

    def rule(self, event):
        return event.get("eventName") in self.DISCOVERY_EVENTS

    def title(self, event):
        return f"User [{event.udm('actor_user')}] performed a [{event.get('eventName')}] action in AWS account [{event.get('recipientAccountId')}]."

    def dedup(self, event):
        return event.udm("actor_user")

    def alert_context(self, event):
        return aws_rule_context(event)

    tests = [
        RuleTest(
            name="Discovery Event Names",
            expected_result=True,
            log={
                "awsRegion": "us-west-2",
                "eventID": "6faccad9-b6ae-4549-8e39-03430cbce2aa",
                "eventName": "DescribeSecurityGroups",
                "eventSource": "ec2.amazonaws.com",
                "eventTime": "2021-10-19 01:06:59",
                "eventType": "AwsApiCall",
                "eventVersion": "1.08",
                "managementEvent": True,
                "p_any_aws_account_ids": ["123456789012"],
                "p_any_aws_arns": [
                    "arn:aws:iam::123456789012:role/ExampleRole-us-east-2",
                    "arn:aws:sts::123456789012:assumed-role/ExampleRole-us-east-2/153151351351351",
                ],
                "p_any_ip_addresses": ["12.34.56.78"],
                "p_event_time": "2021-10-19 01:06:59",
                "p_log_type": "AWS.CloudTrail",
                "p_parse_time": "2021-10-19 01:11:10.412",
                "p_row_id": "eabaceda7842c2b2e7a398f40cc150",
                "p_source_id": "5f9f0f60-9c56-4027-b93a-8bab3019f0f1",
                "p_source_label": "Hosted - Cloudtrail - XYZ",
                "readOnly": True,
                "recipientAccountId": "123456789012",
                "requestID": "43efad11-bb40-43df-ad25-c8e7f0bfdc7a",
                "requestParameters": {
                    "filterSet": {},
                    "securityGroupIdSet": {"items": [{"groupId": "sg-01e29ae063f5f63a0"}]},
                    "securityGroupSet": {},
                },
                "sourceIPAddress": "12.34.56.78",
                "userAgent": "aws-sdk-go/1.40.21 (go1.17; linux; amd64) exec-env/AWS_Lambda_go1.x",
                "userIdentity": {
                    "accessKeyId": "ASIA153151351351351",
                    "accountId": "123456789012",
                    "arn": "arn:aws:sts::123456789012:assumed-role/ExampleRole-us-east-2/153151351351351",
                    "principalId": "AROA4NOI7P47OHH3NQORX:153151351351351",
                    "sessionContext": {
                        "attributes": {"creationDate": "2021-10-19T01:05:18Z", "mfaAuthenticated": "false"},
                        "sessionIssuer": {
                            "accountId": "123456789012",
                            "arn": "arn:aws:iam::123456789012:role/ExampleRole-us-east-2",
                            "principalId": "AROA153151351351351",
                            "type": "Role",
                            "userName": "ExampleRole-us-east-2",
                        },
                        "webIdFederationData": {},
                    },
                    "type": "AssumedRole",
                },
            },
        ),
        RuleTest(
            name="Non Discovery Event Names",
            expected_result=False,
            log={
                "apiVersion": "2012-08-10",
                "awsRegion": "us-east-1",
                "eventCategory": "Data",
                "eventID": "9b666120-d5f9-4ca8-b158-3cd53e4c48bd",
                "eventName": "GetRecords",
                "eventSource": "dynamodb.amazonaws.com",
                "eventTime": "2023-01-09 16:01:21",
                "eventType": "AwsApiCall",
                "eventVersion": "1.08",
                "managementEvent": False,
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class SnowflakeStreamAccountAdminGranted(Rule):
    id = "Snowflake.Stream.AccountAdminGranted-prototype"
    display_name = "Snowflake Account Admin Granted"
    log_types = [LogType.SNOWFLAKE_GRANTS_TO_USERS]
    default_severity = Severity.MEDIUM
    reports = {"MITRE ATT&CK": ["TA0004:T1078"]}
    default_description = "Detect when account admin is granted."
    tags = ["Snowflake", "[MITRE] Privilege Escalation", "[MITRE] Valid Accounts"]

    def rule(self, event):
        if event.get("DELETED_ON"):
            return False
        return "admin" in event.get("GRANTEE_NAME", "").lower()

    def title(self, event):
        source_name = event.get("p_source_label", "<UNKNOWN SNOWFLAKE SOURCE>")
        target = event.get("GRANTED_TO", "<UNKNOWN TARGET>")
        actor = event.get("GRANTED_BY", "<UNKNOWN ACTOR>")
        role = event.get("GRANTEE_NAME", "<UNKNOWN ROLE>")
        return f"{source_name}: {actor} granted role {role} to {target}"

    tests = [
        RuleTest(
            name="Admin Role Assigned",
            expected_result=True,
            log={
                "p_event_time": "2024-10-08 11:24:50.682000000",
                "p_log_type": "Snowflake.GrantsToUsers",
                "p_source_label": "Snowflake Prod",
                "CREATED_ON": "2024-10-08 11:24:50.682000000",
                "GRANTED_BY": "SNOWFLAKE",
                "GRANTED_TO": "APPLICATION_ROLE",
                "GRANTEE_NAME": "TRUST_CENTER_ADMIN",
            },
        ),
        RuleTest(
            name="Non-Admin Role Assigned",
            expected_result=False,
            log={
                "p_event_time": "2024-10-08 11:24:50.682000000",
                "p_log_type": "Snowflake.GrantsToUsers",
                "p_source_label": "Snowflake Prod",
                "CREATED_ON": "2024-10-08 11:24:50.682000000",
                "GRANTED_BY": "SNOWFLAKE",
                "GRANTED_TO": "APPLICATION_ROLE",
                "GRANTEE_NAME": "TRUST_CENTER_VIEWER",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class IntuneNewDeviceManagementScript(Rule):
    id = "Intune.NewDeviceManagementScript-prototype"
    display_name = "Intune New Device Management Script"
    log_types = [LogType.MICROSOFT_INTUNE_AUDIT_LOGS]
    tags = ["InTune"]
    default_severity = Severity.MEDIUM
    reports = {"MITRE ATT&CK": ["TA0002:T1072", "TA0008:T1021.007", "TA0005:T1202"]}
    default_description = "Microsoft Intune allows administrators to deploy scripts to devices as a means of remote management and configuration. These scripts, which can be executed by the local SYSTEM account, provides a powerful capability to managed devices. This functionality can be abused by adversaries to deploy malicious scripts to devices, thereby allowing adversaries to pivot from compromised accounts to endpoints. This detection identifies changes to device management scripts, to include creation, modification, and deletion of scripts."
    default_runbook = "Review the actions taken to determine if they are legitimate changes by an administrator. Microsoft does not provide the IP address of the actor or the name of the affected scripts, but more information can be found by navigating to the Intune portal and reviewing the configured scripts. The TargetObjectIds field specifies the ID of the script(s) that were targeted by the given action.  The TargetGroupIDs specify target groups of devices that the script will be assigned to. If the actor deletes a script, it will not be possible to retrieve more information from the portal. Endpoint investigation can be performed by reviewing InTune logs at C:\\ProgramData\\Microsoft\\IntuneManagementExtension\\Logs\\_IntuneManagementExtension.txt on affected endpoints."
    default_reference = (
        "https://cloud.google.com/blog/topics/threat-intelligence/abusing-intune-permissions-entra-id-environments"
    )
    ACTOR = OPERATION = ""

    def rule(self, event):
        # Alert on DeviceManagementScript or DeviceHealthScript events
        self.OPERATION = event.get("operationName")
        return "DeviceManagementScript" in self.OPERATION or "DeviceComplianceScript" in self.OPERATION

    def title(self, event):
        self.ACTOR = event.get("identity", "")
        # Return a generic title if the operation is unknown
        if self.OPERATION == "Unknown":
            return f"A change to InTune device management scripts was performed by [{self.ACTOR}]."
        # The script type is the second word in the operation
        script_type_parts = self.OPERATION.split(" ")
        if len(script_type_parts) > 1:
            script_type = script_type_parts[1]
        else:
            script_type = "Unknown"
        if self.OPERATION.startswith("create"):
            action = "created"
        elif self.OPERATION.startswith("assign"):
            action = "assigned"
        elif self.OPERATION.startswith("delete"):
            action = "deleted"
        elif self.OPERATION.startswith("patched"):
            action = "patched"
        else:
            action = "unknown"
        return f"An InTune device [{script_type}] script was [{action}] by [{self.ACTOR}]"

    def alert_context(self, event):
        return {
            "Actor": self.ACTOR,
            "Operation": self.OPERATION,
            "Object IDs": event.deep_get("properties", "TargetObjectIds", default="Unknown"),
        }

    tests = [
        RuleTest(
            name="Add Device Management Script",
            expected_result=True,
            log={
                "category": "AuditLogs",
                "correlationId": "74631aa2-a9ea-4b5d-8d48-3e008d092325",
                "identity": "testuser@testdomain.com",
                "operationName": "assignDeviceManagementScript DeviceManagementScript",
                "properties": {
                    "ActivityDate": "4/1/2025 6:42:09 PM",
                    "ActivityResultStatus": 1,
                    "ActivityType": 3,
                    "Actor": {
                        "ActorType": 1,
                        "Application": "5926fc8e-304e-4f59-8bed-58ca97cc39a4",
                        "ApplicationName": "Microsoft Intune portal extension",
                        "IsDelegatedAdmin": False,
                        "ObjectId": "5774e54b-f2ad-41f8-a2d5-4bc356342fa6",
                        "PartnerTenantId": "00000000-0000-0000-0000-000000000000",
                        "UPN": "testuser@testdomain.com",
                        "UserPermissions": ["*"],
                    },
                    "AdditionalDetails": "Key = TargetGroupIDsValue = 36ca05af-f92e-48b6-8f7a-5bf3924a4926\r\nKey = TargetGroupIDsValue = 36ca05af-f92e-48b6-8f7a-5bf3924a4926\r\n",
                    "AuditEventId": "88abb00c-2752-4c2f-b690-00c6c4f2931b",
                    "Category": 3,
                    "TargetDisplayNames": ["<null>"],
                    "TargetObjectIds": ["914232cd-e64b-4ecd-9fe2-ecfc5f1aef89"],
                    "Targets": [
                        {
                            "ModifiedProperties": [
                                {"Name": "TargetGroupIDs", "New": "36ca05af-f92e-48b6-8f7a-5bf3924a4926"},
                                {"Name": "DeviceManagementAPIVersion", "New": "5025-02-18"},
                            ],
                        },
                    ],
                },
                "resultDescription": "None",
                "resultType": "Success",
                "tenantId": "11111111-2222-3333-4444-555555555555",
                "time": "2025-04-01T18:42:09.5769000Z",
            },
        ),
        RuleTest(
            name="Add Device Compliance Script",
            expected_result=True,
            log={
                "category": "AuditLogs",
                "correlationId": "04ee6f15-684f-4d5b-9cb3-9680cf38c4d6",
                "identity": "testuser@testdomain.com",
                "operationName": "createDeviceComplianceScript DeviceComplianceScript",
                "properties": {
                    "ActivityDate": "4/1/2025 7:39:45 PM",
                    "ActivityResultStatus": 1,
                    "ActivityType": 0,
                    "Actor": {
                        "ActorType": 1,
                        "Application": "5926fc8e-304e-4f59-8bed-58ca97cc39a4",
                        "ApplicationName": "Microsoft Intune portal extension",
                        "IsDelegatedAdmin": False,
                        "ObjectId": "5774e54b-f2ad-41f8-a2d5-4bc356342fa6",
                        "PartnerTenantId": "00000000-0000-0000-0000-000000000000",
                        "UPN": "testuser@testdomain.com",
                        "UserPermissions": ["*"],
                    },
                    "AdditionalDetails": "",
                    "AuditEventId": "d66801a7-0893-4ae0-8407-dd871b4e6cfd",
                    "Category": 3,
                    "TargetDisplayNames": ["<null>"],
                    "TargetObjectIds": ["a0cbb43a-211f-45a0-bb42-6ace6857e914"],
                    "Targets": [{"ModifiedProperties": [{"Name": "DeviceManagementAPIVersion", "New": "5025-02-18"}]}],
                },
                "resultDescription": "None",
                "resultType": "Success",
                "tenantId": "11111111-2222-3333-4444-555555555555",
                "time": "2025-04-01T19:39:45.0755000Z",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.crowdstrike_event_streams import audit_keys_dict, cs_alert_context, str_to_list


@panther_managed
class CrowdstrikeIpAllowlistChanged(Rule):
    id = "Crowdstrike.IpAllowlistChanged-prototype"
    display_name = "Crowdstrike IP Allowlist Changed"
    log_types = [LogType.CROWDSTRIKE_EVENT_STREAMS]
    default_severity = Severity.INFO
    create_alert = False
    reports = {"MITRE ATT&CK": ["TA0003:T1556.009", "TA0005:T1556.009", "TA0006:T1556.009"]}
    default_description = "Updates were made to Falcon console's allowlist. This could indicate a bad actor permitting access from another machine, or could be attackers preventing legitimate actors from accessing the console."
    default_runbook = "Validate this action was authorized."

    def rule(self, event):
        # Only alert if an allow list is created or edited
        op_name = event.deep_get("event", "OperationName")
        if op_name not in ("CreateAllowlistGroup", "UpdateAllowlistGroup"):
            return False
        return True

    def title(self, event):
        actor = event.deep_get("event", "UserId")
        action = {"CreateAllowlistGroup": "created a new", "UpdateAllowlistGroup": "made changes to"}.get(
            event.deep_get("event", "OperationName"),
        )
        group = audit_keys_dict(event).get("group_name", "UNKNWOWN GROUP")
        return f"{actor} {action} Crowdstrike IP allowlist group: {group}"

    def alert_context(self, event):
        context = cs_alert_context(event)
        # Be nice and concert the "lists" into actual lists so customers can easily process the alert
        #   context
        for key in ("cidrs", "old_cidrs", "contexts", "old_contexts"):
            if context.get(key):
                try:
                    context[key] = str_to_list(context[key])
                except ValueError:
                    pass  # Just ignore if we can't unmarshal it
        # Find out what entries were removed, and which were added
        op_name = event.deep_get("event", "OperationName")
        audit_keys = audit_keys_dict(event)
        added_cidrs = []
        removed_cidrs = []
        added_contexts = []
        removed_contexts = []

        def getlist(key: str):
            return str_to_list(audit_keys.get(key))

        match op_name:
            case "UpdateAllowlistGroup":
                new_cidrs = getlist("cidrs")
                old_cidrs = getlist("old_cidrs")
                new_ctx = getlist("contexts")
                old_ctx = getlist("old_contexts")
                added_cidrs = self.get_unique_entries(new_cidrs, old_cidrs)
                removed_cidrs = self.get_unique_entries(old_cidrs, new_cidrs)
                added_contexts = self.get_unique_entries(new_ctx, old_ctx)
                removed_contexts = self.get_unique_entries(old_ctx, new_ctx)
            case "CreateAllowlistGroup":
                added_cidrs = str_to_list(audit_keys.get("cidrs", []))
                added_contexts = str_to_list(audit_keys.get("contexts", []))
        context.update(
            {
                "changes": {
                    "cidr_added": added_cidrs,
                    "cidr_removed": removed_cidrs,
                    "context_added": added_contexts,
                    "context_removed": removed_contexts,
                },
            },
        )
        return context

    def get_unique_entries(self, list1: list, list2: list) -> list:
        """Returns items in l1 that are not in l2."""
        return list(set(list1) - set(list2))

    tests = [
        RuleTest(
            name="A Single IP In Created Allowlist",
            expected_result=True,
            log={
                "event": {
                    "AuditKeyValues": [
                        {"Key": "allowlist_group_id", "ValueString": "24821376-7e77-431e-9469-74846978fe64"},
                        {"Key": "group_name", "ValueString": "example_group"},
                        {"Key": "description", "ValueString": ""},
                        {"Key": "cidrs", "ValueString": "[1.1.1.1]"},
                        {"Key": "contexts", "ValueString": "[API]"},
                        {"Key": "active", "ValueString": "false"},
                    ],
                    "OperationName": "CreateAllowlistGroup",
                    "ServiceName": "Crowdstrike Allowlist Management",
                    "Success": True,
                    "UTCTimestamp": "2024-07-26 16:13:13.000000000",
                    "UserId": "wormtongue@isengard.org",
                    "UserIp": "1.2.3.4",
                },
                "metadata": {
                    "customerIDString": "fake_cust_id",
                    "eventCreationTime": "2024-07-26 16:13:13.579000000",
                    "eventType": "AuthActivityAuditEvent",
                    "offset": 365164,
                    "version": "1.0",
                },
            },
        ),
        RuleTest(
            name="Multiple Single IPs In Created Allowlist",
            expected_result=True,
            log={
                "event": {
                    "AuditKeyValues": [
                        {"Key": "allowlist_group_id", "ValueString": "24821376-7e77-431e-9469-74846978fe64"},
                        {"Key": "group_name", "ValueString": "example_group"},
                        {"Key": "description", "ValueString": ""},
                        {"Key": "cidrs", "ValueString": "[1.1.1.1 2.2.2.2 3.3.3.3/32]"},
                        {"Key": "contexts", "ValueString": "[API UI OTHER]"},
                        {"Key": "active", "ValueString": "false"},
                    ],
                    "OperationName": "CreateAllowlistGroup",
                    "ServiceName": "Crowdstrike Allowlist Management",
                    "Success": True,
                    "UTCTimestamp": "2024-07-26 16:13:13.000000000",
                    "UserId": "wormtongue@isengard.org",
                    "UserIp": "1.2.3.4",
                },
                "metadata": {
                    "customerIDString": "fake_cust_id",
                    "eventCreationTime": "2024-07-26 16:13:13.579000000",
                    "eventType": "AuthActivityAuditEvent",
                    "offset": 365164,
                    "version": "1.0",
                },
            },
        ),
        RuleTest(
            name="Single IP Added to existing Allowlist",
            expected_result=True,
            log={
                "event": {
                    "AuditKeyValues": [
                        {"Key": "old_group_name", "ValueString": "my_allowlist"},
                        {"Key": "old_cidrs", "ValueString": "[1.2.3.4/8]"},
                        {"Key": "allowlist_group_id", "ValueString": "24821376-7e77-431e-9469-74846978fe64"},
                        {"Key": "group_name", "ValueString": "my_allowlist"},
                        {"Key": "description", "ValueString": ""},
                        {"Key": "cidrs", "ValueString": "[1.2.3.4/8 32.32.32.32]"},
                        {"Key": "contexts", "ValueString": "[API]"},
                        {"Key": "active", "ValueString": "false"},
                        {"Key": "old_allowlist_group_id", "ValueString": "24821376-7e77-431e-9469-74846978fe64"},
                        {"Key": "old_description", "ValueString": ""},
                        {"Key": "old_contexts", "ValueString": "[API]"},
                        {"Key": "old_active", "ValueString": "false"},
                    ],
                    "OperationName": "UpdateAllowlistGroup",
                    "ServiceName": "Crowdstrike Allowlist Management",
                    "Success": True,
                    "UTCTimestamp": "2024-07-26 19:47:16.000000000",
                    "UserId": "wormtongue@isengard.org",
                    "UserIp": "1.2.3.4",
                },
                "metadata": {
                    "customerIDString": "fake_customer_id",
                    "eventCreationTime": "2024-07-26 19:47:16.428000000",
                    "eventType": "AuthActivityAuditEvent",
                    "offset": 366148,
                    "version": "1.0",
                },
            },
        ),
        RuleTest(
            name="CIDR Removed from existing Allowlist",
            expected_result=True,
            log={
                "event": {
                    "AuditKeyValues": [
                        {"Key": "old_group_name", "ValueString": "my_allowlist"},
                        {"Key": "old_cidrs", "ValueString": "[1.2.3.4/8 8.8.8.8/12]"},
                        {"Key": "allowlist_group_id", "ValueString": "24821376-7e77-431e-9469-74846978fe64"},
                        {"Key": "group_name", "ValueString": "my_allowlist"},
                        {"Key": "description", "ValueString": ""},
                        {"Key": "cidrs", "ValueString": "[1.2.3.4/8]"},
                        {"Key": "contexts", "ValueString": "[API]"},
                        {"Key": "active", "ValueString": "false"},
                        {"Key": "old_allowlist_group_id", "ValueString": "24821376-7e77-431e-9469-74846978fe64"},
                        {"Key": "old_description", "ValueString": ""},
                        {"Key": "old_contexts", "ValueString": "[API]"},
                        {"Key": "old_active", "ValueString": "false"},
                    ],
                    "OperationName": "UpdateAllowlistGroup",
                    "ServiceName": "Crowdstrike Allowlist Management",
                    "Success": True,
                    "UTCTimestamp": "2024-07-26 19:47:16.000000000",
                    "UserId": "wormtongue@isengard.org",
                    "UserIp": "1.2.3.4",
                },
                "metadata": {
                    "customerIDString": "fake_customer_id",
                    "eventCreationTime": "2024-07-26 19:47:16.428000000",
                    "eventType": "AuthActivityAuditEvent",
                    "offset": 366148,
                    "version": "1.0",
                },
            },
        ),
        RuleTest(
            name="Only CIDR Ranges In Created Allowlist",
            expected_result=True,
            log={
                "event": {
                    "AuditKeyValues": [
                        {"Key": "allowlist_group_id", "ValueString": "24821376-7e77-431e-9469-74846978fe64"},
                        {"Key": "group_name", "ValueString": "example_group"},
                        {"Key": "description", "ValueString": ""},
                        {"Key": "cidrs", "ValueString": "[1.1.1.1/12 2.2.2.2/8 3.3.3.3/4]"},
                        {"Key": "contexts", "ValueString": "[API UI OTHER]"},
                        {"Key": "active", "ValueString": "false"},
                    ],
                    "OperationName": "CreateAllowlistGroup",
                    "ServiceName": "Crowdstrike Allowlist Management",
                    "Success": True,
                    "UTCTimestamp": "2024-07-26 16:13:13.000000000",
                    "UserId": "wormtongue@isengard.org",
                    "UserIp": "1.2.3.4",
                },
                "metadata": {
                    "customerIDString": "fake_cust_id",
                    "eventCreationTime": "2024-07-26 16:13:13.579000000",
                    "eventType": "AuthActivityAuditEvent",
                    "offset": 365164,
                    "version": "1.0",
                },
            },
        ),
        RuleTest(
            name="Unrelated Event",
            expected_result=False,
            log={
                "metadata": {
                    "customerIDString": "fake_customer_id",
                    "offset": 341329,
                    "eventType": "AuthActivityAuditEvent",
                    "eventCreationTime": "2024-07-22 15:50:16.923000000",
                    "version": "1.0",
                },
                "event": {
                    "UserId": "sharkey@hobbiton.co",
                    "UserIp": "192.0.2.100",
                    "OperationName": "deleteUser",
                    "ServiceName": "CrowdStrike Authentication",
                    "Success": True,
                    "UTCTimestamp": "2024-07-22 15:50:16.923000000",
                    "AuditKeyValues": [{"Key": "target_name", "ValueString": "frodo.baggins@hobbiton.co"}],
                },
            },
        ),
    ]



import ipaddress

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_rule_context


@panther_managed
class AWSCloudTrailVPCES3ExternalIP(Rule):
    id = "AWS.CloudTrail.VPCE.S3ExternalIP-prototype"
    display_name = "S3 Access Via VPC Endpoint From External IP"
    log_types = [LogType.AWS_CLOUDTRAIL]
    default_severity = Severity.MEDIUM
    tags = [
        "AWS",
        "VPC",
        "S3",
        "Data Exfiltration",
        "CloudTrail",
        "Network Boundary Bridging",
        "Exfiltration Over Alternative Protocol",
    ]
    default_description = "Detects S3 data access through VPC endpoints from external/public IP addresses, which could indicate data exfiltration attempts.\n\nThis rule can be customized with the following overrides:\n- S3_DATA_ACCESS_OPERATIONS: List of S3 operations to monitor\n"
    default_runbook = "1. Identify the principal and the specific S3 objects being accessed\n2. Verify if the external IP address belongs to a legitimate service or entity\n3. Check if the access pattern is expected for this user/role\n4. Review the contents of the S3 objects to determine sensitivity\n5. If unauthorized, determine how the principal obtained access credentials\n6. Revoke access immediately if determined to be malicious\n7. Consider implementing stricter bucket policies and VPC endpoint policies\n"
    default_reference = "https://www.wiz.io/blog/aws-vpc-endpoint-cloudtrail"
    summary_attributes = [
        "userIdentity.principalId",
        "sourceIPAddress",
        "eventSource",
        "eventName",
        "requestParameters",
        "resources",
    ]
    # Define S3 data access operations
    S3_DATA_ACCESS_OPERATIONS = [
        "GetObject",
        "GetObjectVersion",
        "GetObjectAcl",
        "GetObjectVersionAcl",
        "PutObject",
        "PutObjectAcl",
        "PutObjectVersionAcl",
        "CopyObject",
        "DeleteObject",
        "DeleteObjects",
        "DeleteObjectVersion",
    ]

    def rule(self, event):
        # Check if this is a VPC Endpoint network activity event for S3
        if (
            event.get("eventType") != "AwsVpceEvent"
            or event.get("eventCategory") != "NetworkActivity"
            or event.get("eventSource") != "s3.amazonaws.com"
        ):
            return False
        # Focus on data access operations
        if event.get("eventName") not in self.S3_DATA_ACCESS_OPERATIONS:
            return False
        # Check for external IP
        source_ip = event.get("sourceIPAddress", "")
        if not source_ip:
            return False
        try:
            ip_obj = ipaddress.ip_address(source_ip)
            if ip_obj.is_global:
                return True
        except ValueError:
            # If source_ip is not a valid IP address
            pass
        return False

    def title(self, event):
        # Use UDM actor_user which leverages the get_actor_user helper function
        actor_user = event.udm("actor_user")
        source_ip = event.get("sourceIPAddress", "unknown")
        bucket_name = event.deep_get("requestParameters", "bucketName", default="unknown")
        return (
            f"S3 Access via VPC Endpoint from External IP: [{actor_user}] from [{source_ip}] to bucket [{bucket_name}]"
        )

    def alert_context(self, event):
        account_id = event.deep_get("userIdentity", "accountId", default="")
        context = aws_rule_context(event)
        context.update(
            {
                "account_id": account_id,
                "principal_id": event.deep_get("userIdentity", "principalId", default="unknown"),
                "actor_user": event.udm("actor_user"),
                "source_ip": event.get("sourceIPAddress", "unknown"),
                "event_source": event.get("eventSource", "unknown"),
                "api_call": event.get("eventName", "unknown"),
                "resources": event.get("resources", []),
                "request_parameters": event.get("requestParameters", {}),
                "config": {"operations_monitored": self.S3_DATA_ACCESS_OPERATIONS},
            },
        )
        return context

    tests = [
        RuleTest(
            name="External IP Access",
            expected_result=True,
            log={
                "eventType": "AwsVpceEvent",
                "eventCategory": "NetworkActivity",
                "eventSource": "s3.amazonaws.com",
                "eventName": "GetObject",
                "userIdentity": {"type": "IAMUser", "principalId": "AIDAEXAMPLE", "accountId": "012345678901"},
                "sourceIPAddress": "8.8.8.8",
                "requestParameters": {"bucketName": "sensitive-data-bucket", "key": "confidential/file.pdf"},
                "resources": [
                    {"type": "AWS::S3::Object", "ARN": "arn:aws:s3:::sensitive-data-bucket/confidential/file.pdf"},
                ],
                "eventTime": "2023-01-01T12:00:00Z",
                "vpcEndpointId": "vpce-EXAMPLE08c1b6b9b7",
                "vpcEndpointAccountId": "012345678901",
                "recipientAccountId": "012345678901",
            },
        ),
        RuleTest(
            name="Internal IP Access",
            expected_result=False,
            log={
                "eventType": "AwsVpceEvent",
                "eventCategory": "NetworkActivity",
                "eventSource": "s3.amazonaws.com",
                "eventName": "GetObject",
                "userIdentity": {"type": "IAMUser", "principalId": "AIDAEXAMPLE", "accountId": "012345678901"},
                "sourceIPAddress": "10.0.0.1",
                "requestParameters": {"bucketName": "sensitive-data-bucket", "key": "confidential/file.pdf"},
                "resources": [
                    {"type": "AWS::S3::Object", "ARN": "arn:aws:s3:::sensitive-data-bucket/confidential/file.pdf"},
                ],
                "eventTime": "2023-01-01T12:00:00Z",
                "vpcEndpointId": "vpce-EXAMPLE08c1b6b9b7",
                "vpcEndpointAccountId": "012345678901",
                "recipientAccountId": "012345678901",
            },
        ),
        RuleTest(
            name="Not S3 Service",
            expected_result=False,
            log={
                "eventType": "AwsVpceEvent",
                "eventCategory": "NetworkActivity",
                "eventSource": "ec2.amazonaws.com",
                "eventName": "DescribeInstances",
                "userIdentity": {"type": "IAMUser", "principalId": "AIDAEXAMPLE", "accountId": "012345678901"},
                "sourceIPAddress": "8.8.8.8",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.tines import tines_alert_context


@panther_managed
class TinesEnqueuedRetryingJobDestruction(Rule):
    id = "Tines.Enqueued.Retrying.Job.Destruction-prototype"
    display_name = "Tines Enqueued/Retrying Job Deletion"
    log_types = [LogType.TINES_AUDIT]
    tags = ["Tines"]
    default_severity = Severity.LOW
    default_description = "Currently enqueued or retrying jobs were cleared"
    default_runbook = (
        "Possible data destruction. Please reach out to the user and confirm this was done for valid business reasons."
    )
    default_reference = "https://www.tines.com/docs/self-hosting/job-management"

    def rule(self, event):
        return event.get("operation_name", "<NO_OPERATION_NAME>") in ["JobsQueuedDeletion", "JobsRetryingDeletion"]

    def title(self, event):
        operation = event.get("operation_name", "<NO_OPERATION_NAME>")
        user = event.get("user_email", "<NO_USER_EMAIL>")
        tines_instance = event.get("p_source_label", "<NO_SOURCE_LABEL>")
        return f"Tines [{operation}] performed by [{user}] on [{tines_instance}]."

    def alert_context(self, event):
        return tines_alert_context(event)

    tests = [
        RuleTest(
            name="Trigger - JobsQueuedDeletion",
            expected_result=True,
            log={
                "created_at": "2023-06-13 15:14:46",
                "id": 1234,
                "operation_name": "JobsQueuedDeletion",
                "p_source_label": "tines-log-source-name",
                "request_ip": "98.224.225.84",
                "request_user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (  KHTML, like Gecko) Chrome/114.0.0.0 Safari/537.36",
                "tenant_id": "1337",
                "user_email": "user@email.com",
                "user_id": "7331",
                "user_name": "Tines User Person",
            },
        ),
        RuleTest(
            name="Trigger - JobsRetryingDeletion",
            expected_result=True,
            log={
                "created_at": "2023-06-13 15:14:46",
                "id": 1234,
                "operation_name": "JobsRetryingDeletion",
                "p_source_label": "tines-log-source-name",
                "request_ip": "98.224.225.84",
                "request_user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (  KHTML, like Gecko) Chrome/114.0.0.0 Safari/537.36",
                "tenant_id": "1337",
                "user_email": "user@email.com",
                "user_id": "7331",
                "user_name": "Tines User Person",
            },
        ),
        RuleTest(
            name="Tines Login",
            expected_result=False,
            log={
                "created_at": "2023-05-17 14:45:19",
                "id": 7888888,
                "operation_name": "Login",
                "p_source_label": "tines-log-source-name",
                "request_ip": "12.12.12.12",
                "request_user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                "tenant_id": "8888",
                "user_email": "user@company.com",
                "user_id": "17171",
                "user_name": "user at company dot com",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class AWSBedrockModelInvocationAbnormalTokenUsage(Rule):
    id = "AWS.BedrockModelInvocation.AbnormalTokenUsage-prototype"
    display_name = "AWS Bedrock Model Invocation Abnormal Token Usage"
    log_types = [LogType.AWS_BEDROCK_MODEL_INVOCATION]
    tags = ["AWS", "Bedrock", "Beta", "Resource Hijacking"]
    default_severity = Severity.INFO
    reports = {"MITRE ATT&CK": ["TA0040:T1496.004"]}
    default_description = "Monitors for potential misuse or abuse of AWS Bedrock AI models by detecting abnormal token usage patterns and alerts when the total token usage exceeds the appropriate threshold for each different type of model."
    default_runbook = "Verify the alert details by checking token usage, model ID, and account information to confirm unusual activity, examine user access patterns to identify potential credential compromise, and look for evidence of prompt injection, unusual repetition, or attempts to bypass usage limits. Apply stricter usage quotas to the affected account, block suspicious IP addresses, and enhance the guardrails that are in place."
    default_reference = "https://stratus-red-team.cloud/attack-techniques/AWS/aws.impact.bedrock-invoke-model/"
    summary_attributes = ["p_any_aws_account_ids", "p_any_aws_arns"]
    inline_filters = [{"All": []}]

    def rule(self, event):
        # Only process InvokeModel and Converse operations
        if event.get("operation") not in ["InvokeModel", "Converse"]:
            return False
        # retrieve the necessary values from the logs
        token_usage = event.deep_get("output", "outputBodyJson", "usage", "totalTokens", default=0)
        model_id = event.get("modelId", default="")
        # Get the appropriate threshold for each model
        if "haiku" in model_id:
            threshold = 3000
        elif "sonnet" in model_id:
            threshold = 4000
        elif "opus" in model_id:
            threshold = 5000
        else:
            threshold = 4000  # default threshold
        # Check for abnormal token usage
        if token_usage > threshold:
            return True
        # Flag unusual token patterns (high usage with no actual output)
        output_tokens = event.deep_get("output", "outputBodyJson", "usage", "outputTokens", default=0)
        if token_usage > 1000 and output_tokens == 0:
            return True
        return False

    def title(self, event):
        model_id = event.get("modelId", default="unknown")
        operation_name = event.get("operation", default="unknown")
        account_id = event.get("accountId", default="unknown")
        token_usage = event.deep_get("output", "outputBodyJson", "usage", "totalTokens", default=0)
        title_parts = [
            f"Abnormal token usage detected: {token_usage} tokens",
            f"Model: {model_id}",
            f"Operation: {operation_name}",
            f"Account: {account_id}",
        ]
        return " | ".join(title_parts)

    tests = [
        RuleTest(
            name="Converse Operation Unusual Token Patterns",
            expected_result=True,
            log={
                "accountId": "111111111111",
                "identity": {"arn": "arn:aws:sts::111111111111:assumed-role/role_details/suspicious.user"},
                "input": {
                    "inputBodyJson": {
                        "messages": [{"content": [{"text": "I have a very suspicious question."}], "role": "user"}],
                    },
                    "inputContentType": "application/json",
                    "inputTokenCount": 0,
                },
                "modelId": "anthropic.claude-3-haiku-20240307-v1:0",
                "operation": "Converse",
                "output": {
                    "outputBodyJson": {
                        "metrics": {"latencyMs": 249},
                        "output": {
                            "message": {"content": [{"text": "You shouldn't ask this question"}], "role": "assistant"},
                        },
                        "usage": {"inputTokens": 0, "outputTokens": 0, "totalTokens": 2000},
                    },
                    "outputContentType": "application/json",
                    "outputTokenCount": 0,
                },
                "region": "us-west-2",
                "requestId": "bb98d9a8-bd9a-47ca-976b-f165ef1f8b67",
                "schemaType": "ModelInvocationLog",
                "schemaVersion": "1.0",
                "timestamp": "2025-05-15 14:17:22.000000000",
            },
        ),
        RuleTest(
            name="Converse Operation with Abnormal Token Usage",
            expected_result=True,
            log={
                "accountId": "111111111111",
                "identity": {"arn": "arn:aws:sts::111111111111:assumed-role/role_details/suspicious.user"},
                "input": {
                    "inputBodyJson": {
                        "messages": [{"content": [{"text": "I have a very suspicious question."}], "role": "user"}],
                    },
                    "inputContentType": "application/json",
                    "inputTokenCount": 0,
                },
                "modelId": "anthropic.claude-3-haiku-20240307-v1:0",
                "operation": "Converse",
                "output": {
                    "outputBodyJson": {
                        "metrics": {"latencyMs": 249},
                        "output": {
                            "message": {"content": [{"text": "You shouldn't ask this question"}], "role": "assistant"},
                        },
                        "usage": {"inputTokens": 0, "outputTokens": 0, "totalTokens": 5000},
                    },
                    "outputContentType": "application/json",
                    "outputTokenCount": 0,
                },
                "region": "us-west-2",
                "requestId": "bb98d9a8-bd9a-47ca-976b-f165ef1f8b67",
                "schemaType": "ModelInvocationLog",
                "schemaVersion": "1.0",
                "timestamp": "2025-05-15 14:17:22.000000000",
            },
        ),
        RuleTest(
            name="Perform Another Operation",
            expected_result=False,
            log={
                "accountId": "111111111111",
                "identity": {"arn": "arn:aws:sts::111111111111:assumed-role/role_details/regular.user"},
                "input": {
                    "inputBodyJson": {
                        "messages": [{"content": [{"text": "I have a rather normal question."}], "role": "user"}],
                    },
                    "inputContentType": "application/json",
                    "inputTokenCount": 0,
                },
                "modelId": "anthropic.claude-3-haiku-20240307-v1:0",
                "operation": "ListModels",
                "output": {
                    "outputBodyJson": {
                        "metrics": {"latencyMs": 249},
                        "output": {
                            "message": {"content": [{"text": "I can respond to this question"}], "role": "assistant"},
                        },
                        "usage": {"inputTokens": 0, "outputTokens": 0, "totalTokens": 0},
                    },
                    "outputContentType": "application/json",
                    "outputTokenCount": 0,
                },
                "region": "us-west-2",
                "requestId": "bb98d9a8-bd9a-47ca-976b-f165ef1f8b67",
                "schemaType": "ModelInvocationLog",
                "schemaVersion": "1.0",
                "timestamp": "2025-05-15 14:17:22.000000000",
            },
        ),
        RuleTest(
            name="Regular Converse Operation with Normal Token Usage",
            expected_result=False,
            log={
                "accountId": "111111111111",
                "identity": {"arn": "arn:aws:sts::111111111111:assumed-role/role_details/regular.user"},
                "input": {
                    "inputBodyJson": {
                        "messages": [{"content": [{"text": "I have a rather normal question."}], "role": "user"}],
                    },
                    "inputContentType": "application/json",
                    "inputTokenCount": 0,
                },
                "modelId": "anthropic.claude-3-haiku-20240307-v1:0",
                "operation": "Converse",
                "output": {
                    "outputBodyJson": {
                        "metrics": {"latencyMs": 249},
                        "output": {
                            "message": {"content": [{"text": "I can respond to this question"}], "role": "assistant"},
                        },
                        "usage": {"inputTokens": 0, "outputTokens": 0, "totalTokens": 0},
                    },
                    "outputContentType": "application/json",
                    "outputTokenCount": 0,
                },
                "region": "us-west-2",
                "requestId": "bb98d9a8-bd9a-47ca-976b-f165ef1f8b67",
                "schemaType": "ModelInvocationLog",
                "schemaVersion": "1.0",
                "timestamp": "2025-05-15 14:17:22.000000000",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.crowdstrike_fdr import crowdstrike_detection_alert_context


@panther_managed
class CrowdstrikeCryptominingTools(Rule):
    default_description = "Detects the execution of known crytocurrency mining tools."
    display_name = "Crowdstrike Cryptomining Tools "
    default_reference = "https://www.crowdstrike.com/cybersecurity-101/cryptojacking/"
    default_severity = Severity.CRITICAL
    log_types = [LogType.CROWDSTRIKE_FDR_EVENT]
    id = "Crowdstrike.Cryptomining.Tools-prototype"
    CRYPTOCURRENCY_MINING_TOOLS = {
        "xmrig.exe",
        "cgminer.exe",
        "bfgminer.exe",
        "ethminer.exe",
        "minergate-cli.exe",
        "nicehashminer.exe",
        "sgminer.exe",
        "cpuminer.exe",
        "cudaminer.exe",
        "nheqminer.exe",
        "claymore.exe",
        "xmr-stak.exe",
        "ccminer.exe",
        "zecminer64.exe",
        "ewbf.exe",
        "nanominer.exe",
        "phoenixminer.exe",
        "t-rex.exe",
        "gminer.exe",
        "lolminer.exe",
        "teamredminer.exe",
        "nbminer.exe",
        "daggerhashimoto.exe",
    }

    def rule(self, event):
        if event.get("fdr_event_type", "") == "ProcessRollup2":
            if event.get("event_platform", "") == "Win":
                process_name = event.deep_get("event", "ImageFileName", default="").lower().split("\\")[-1]
                return process_name in self.CRYPTOCURRENCY_MINING_TOOLS
        return False

    def title(self, event):
        tool = event.deep_get("event", "ImageFileName", default="<TOOL_NOT_FOUND>").lower().split("\\")[-1]
        host = event.get("ComputerName") or event.get("aid", "<AID_NOT_FOUND>")
        return f"Crowdstrike: Cryptocurrency mining tool [{tool}] detected on host [{host}]"

    def alert_context(self, event):
        return crowdstrike_detection_alert_context(event)

    tests = [
        RuleTest(
            name="Crypto tool",
            expected_result=True,
            log={
                "aid": "1234567890abcdefghijklmnop9876",
                "aip": "11.10.9.8",
                "cid": "abcdefghijklmnop123467890",
                "configbuild": "1007.3.0016606.11",
                "configstatehash": "3799024366",
                "entitlements": "15",
                "event": {
                    "AuthenticationId": "293628",
                    "AuthenticodeHashData": "5540c470218d209b7c3eca3d12e190580814d566",
                    "CommandLine": "C:\\Windows\\System32\\ethminer.exe",
                    "ConfigBuild": "1007.3.0016606.11",
                    "ConfigStateHash": "3799024366",
                    "EffectiveTransmissionClass": "2",
                    "Entitlements": "15",
                    "ImageFileName": "\\Device\\HarddiskVolume2\\Windows\\System32\\ethminer.exe",
                    "ImageSubsystem": "3",
                    "IntegrityLevel": "12288",
                    "MD5HashData": "5fd22b915c232378e567160d641cc9f2",
                    "ParentAuthenticationId": "293628",
                    "ParentBaseFileName": "pwsh.exe",
                    "ParentProcessId": "4370948876",
                    "ProcessCreateFlags": "0",
                    "ProcessEndTime": "",
                    "ProcessParameterFlags": "24577",
                    "ProcessStartTime": "1682106752.006",
                    "ProcessSxsFlags": "64",
                    "RawProcessId": "1468",
                    "SHA1HashData": "0000000000000000000000000000000000000000",
                    "SHA256HashData": "488e74e2026d03f21b33f470c23b3de2f466643186c2e06ae7b4883cc2e59377",
                    "SessionId": "2",
                    "SignInfoFlags": "8683538",
                    "SourceProcessId": "4370948876",
                    "SourceThreadId": "6364981533",
                    "Tags": "25, 27, 40, 151, 874, 924, 12094627905582, 12094627906234, 237494511599633",
                    "TargetProcessId": "4390327988",
                    "TokenType": "1",
                    "TreeId": "4295752857",
                    "UserSid": "S-1-5-21-239183934-720705223-383019856-500",
                    "aid": "1234567890abcdefghijklmnop9876",
                    "aip": "11.10.9.8",
                    "cid": "abcdefghijklmnop123467890",
                    "event_platform": "Win",
                    "event_simpleName": "ProcessRollup2",
                    "id": "081d64d7-17fb-40c0-8767-48ff1e2ee2dd",
                    "name": "ProcessRollup2V19",
                    "timestamp": "1682106752722",
                },
                "event_platform": "Win",
                "event_simplename": "ProcessRollup2",
                "fdr_event_type": "ProcessRollup2",
                "id": "081d64d7-17fb-40c0-8767-48ff1e2ee2dd",
                "name": "ProcessRollup2V19",
                "p_any_ip_addresses": ["11.10.9.8"],
                "p_any_md5_hashes": [
                    "5fd22b915c232378e567160d641cc9f2",
                    "1234567890abcdefghijklmnop9876",
                    "abcdefghijklmnop123467890",
                ],
                "p_any_sha1_hashes": ["0000000000000000000000000000000000000000"],
                "p_any_sha256_hashes": ["488e74e2026d03f21b33f470c23b3de2f466643186c2e06ae7b4883cc2e59377"],
                "p_any_trace_ids": ["4295752857", "1234567890abcdefghijklmnop9876", "abcdefghijklmnop123467890"],
                "p_event_time": "2023-04-21 19:52:32.722",
                "p_log_type": "Crowdstrike.FDREvent",
                "p_parse_time": "2023-04-21 20:05:52.94",
                "p_row_id": "7ac82dbb43a99bfec196bdda178c8101",
                "p_schema_version": 0,
                "p_source_id": "1f33f64c-124d-413c-a9e3-d51ccedd8e77",
                "p_source_label": "Crowdstrike-FDR-Dev",
                "timestamp": "2023-04-21 19:52:32.722",
                "treeid": "4295752857",
            },
        ),
        RuleTest(
            name="Other",
            expected_result=False,
            log={
                "aid": "1234567890abcdefghijklmnop9876",
                "aip": "11.10.9.8",
                "cid": "abcdefghijklmnop123467890",
                "configbuild": "1007.3.0016606.11",
                "configstatehash": "3799024366",
                "entitlements": "15",
                "event": {
                    "AuthenticationId": "293628",
                    "AuthenticodeHashData": "5540c470218d209b7c3eca3d12e190580814d566",
                    "CommandLine": '"C:\\Windows\\System32\\at.exe" at 09:00 /interactive /every:m,t,w,th,f,s,su',
                    "ConfigBuild": "1007.3.0016606.11",
                    "ConfigStateHash": "3799024366",
                    "EffectiveTransmissionClass": "2",
                    "Entitlements": "15",
                    "ImageFileName": "\\Device\\HarddiskVolume2\\Windows\\System32\\at.exe",
                    "ImageSubsystem": "3",
                    "IntegrityLevel": "12288",
                    "MD5HashData": "5fd22b915c232378e567160d641cc9f2",
                    "ParentAuthenticationId": "293628",
                    "ParentBaseFileName": "pwsh.exe",
                    "ParentProcessId": "4370948876",
                    "ProcessCreateFlags": "0",
                    "ProcessEndTime": "",
                    "ProcessParameterFlags": "24577",
                    "ProcessStartTime": "1682106752.006",
                    "ProcessSxsFlags": "64",
                    "RawProcessId": "1468",
                    "SHA1HashData": "0000000000000000000000000000000000000000",
                    "SHA256HashData": "488e74e2026d03f21b33f470c23b3de2f466643186c2e06ae7b4883cc2e59377",
                    "SessionId": "2",
                    "SignInfoFlags": "8683538",
                    "SourceProcessId": "4370948876",
                    "SourceThreadId": "6364981533",
                    "Tags": "25, 27, 40, 151, 874, 924, 12094627905582, 12094627906234, 237494511599633",
                    "TargetProcessId": "4390327988",
                    "TokenType": "1",
                    "TreeId": "4295752857",
                    "UserSid": "S-1-5-21-239183934-720705223-383019856-500",
                    "aid": "1234567890abcdefghijklmnop9876",
                    "aip": "11.10.9.8",
                    "cid": "abcdefghijklmnop123467890",
                    "event_platform": "Win",
                    "event_simpleName": "ProcessRollup2",
                    "id": "081d64d7-17fb-40c0-8767-48ff1e2ee2dd",
                    "name": "ProcessRollup2V19",
                    "timestamp": "1682106752722",
                },
                "event_platform": "Win",
                "event_simplename": "ProcessRollup2",
                "fdr_event_type": "ProcessRollup2",
                "id": "081d64d7-17fb-40c0-8767-48ff1e2ee2dd",
                "name": "ProcessRollup2V19",
                "p_any_ip_addresses": ["11.10.9.8"],
                "p_any_md5_hashes": [
                    "5fd22b915c232378e567160d641cc9f2",
                    "1234567890abcdefghijklmnop9876",
                    "abcdefghijklmnop123467890",
                ],
                "p_any_sha1_hashes": ["0000000000000000000000000000000000000000"],
                "p_any_sha256_hashes": ["488e74e2026d03f21b33f470c23b3de2f466643186c2e06ae7b4883cc2e59377"],
                "p_any_trace_ids": ["4295752857", "1234567890abcdefghijklmnop9876", "abcdefghijklmnop123467890"],
                "p_event_time": "2023-04-21 19:52:32.722",
                "p_log_type": "Crowdstrike.FDREvent",
                "p_parse_time": "2023-04-21 20:05:52.94",
                "p_row_id": "7ac82dbb43a99bfec196bdda178c8101",
                "p_schema_version": 0,
                "p_source_id": "1f33f64c-124d-413c-a9e3-d51ccedd8e77",
                "p_source_label": "Crowdstrike-FDR-Dev",
                "timestamp": "2023-04-21 19:52:32.722",
                "treeid": "4295752857",
            },
        ),
    ]



from pypanther.rules.notion.notion_account_changed import NotionAccountChange as NotionAccountChange
from pypanther.rules.notion.notion_login import NotionLogin as NotionLogin
from pypanther.rules.notion.notion_login_from_blocked_ip import NotionLoginFromBlockedIP as NotionLoginFromBlockedIP
from pypanther.rules.notion.notion_login_from_new_location import (
    NotionLoginFromNewLocation as NotionLoginFromNewLocation,
)
from pypanther.rules.notion.notion_many_pages_exported import NotionManyPagesExported as NotionManyPagesExported
from pypanther.rules.notion.notion_page_accessible_to_api import (
    NotionPagePermsAPIPermsChanged as NotionPagePermsAPIPermsChanged,
)
from pypanther.rules.notion.notion_page_accessible_to_guests import (
    NotionPagePermsGuestPermsChanged as NotionPagePermsGuestPermsChanged,
)
from pypanther.rules.notion.notion_page_shared_to_web import NotionPageSharedToWeb as NotionPageSharedToWeb
from pypanther.rules.notion.notion_scim_token_generated import (
    NotionWorkspaceSCIMTokenGenerated as NotionWorkspaceSCIMTokenGenerated,
)
from pypanther.rules.notion.notion_sharing_settings_updated import (
    NotionSharingSettingsUpdated as NotionSharingSettingsUpdated,
)
from pypanther.rules.notion.notion_teamspace_owner_added import NotionTeamspaceOwnerAdded as NotionTeamspaceOwnerAdded
from pypanther.rules.notion.notion_workspace_audit_log_exported import NotionAuditLogExported as NotionAuditLogExported
from pypanther.rules.notion.notion_workspace_exported import NotionWorkspaceExported as NotionWorkspaceExported
from pypanther.rules.notion.notion_workspace_settings_enforce_saml_sso_config_updated import (
    NotionSAMLSSOConfigurationChanged as NotionSAMLSSOConfigurationChanged,
)
from pypanther.rules.notion.notion_workspace_settings_public_homepage_added import (
    NotionWorkspacePublicPageAdded as NotionWorkspacePublicPageAdded,
)



from pypanther.rules.axonius.axonius_add_external_user import AxoniusAddExternalUser as AxoniusAddExternalUser
from pypanther.rules.axonius.axonius_resetapikey import AxoniusResetApiKey as AxoniusResetApiKey
from pypanther.rules.axonius.axonius_too_many_failed_logins import (
    AxoniusTooManyFailedLogins as AxoniusTooManyFailedLogins,
)
from pypanther.rules.axonius.axonius_tor_login import AxoniusTorLogin as AxoniusTorLogin
from pypanther.rules.axonius.axonius_webhook_created import AxoniusWebhookCreated as AxoniusWebhookCreated



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class GCPIAMOrgFolderIAMChanges(Rule):
    id = "GCP.IAM.OrgFolderIAMChanges-prototype"
    display_name = "GCP Org or Folder Policy Was Changed Manually"
    dedup_period_minutes = 1440
    log_types = [LogType.GCP_AUDIT_LOG]
    tags = [
        "GCP",
        "Identity & Access Management",
        "Persistence",
        "Modify Authentication Process - Conditional Access Policies",
    ]
    reports = {"GCP_CIS_1.3": ["1.8"], "MITRE ATT&CK": ["TA0003:T1556.009"]}
    default_severity = Severity.HIGH
    default_description = "Alert if a GCP Org or Folder Policy Was Changed Manually.\n"
    default_runbook = "Contact the party that made the change. If it was intended to be temporary, ask for a window for rollback (< 24 hours). If it must be permanent, ask for change-management doc explaining why it was needed. Direct them to make the change in Terraform to avoid automated rollback. Grep for google_org and google_folder in terraform repos for places to put your new policy bindings.\n"
    default_reference = "https://cloud.google.com/iam/docs/granting-changing-revoking-access"
    summary_attributes = ["severity", "p_any_ip_addresses"]

    def rule(self, event):
        # Return True to match the log event and trigger an alert.
        logname = event.get("logName")
        return (
            event.deep_get("protoPayload", "methodName") == "SetIamPolicy"
            and (logname.startswith("organizations") or logname.startswith("folder"))
            and logname.endswith("/logs/cloudaudit.googleapis.com%2Factivity")
        )

    def title(self, event):
        # use unified data model field in title
        return f"{event.get('p_log_type')}: [{event.udm('actor_user')}] made manual changes to Org policy"

    def alert_context(self, event):
        return {
            "actor": event.udm("actor_user"),
            "policy_change": event.deep_get("protoPayload", "serviceData", "policyDelta"),
            "caller_ip": event.deep_get("protoPayload", "requestMetadata", "callerIP"),
            "user_agent": event.deep_get("protoPayload", "requestMetadata", "callerSuppliedUserAgent"),
        }

    def severity(self, event):
        if event.deep_get("protoPayload", "requestMetadata", "callerSuppliedUserAgent").lower().find("terraform") != -1:
            return "INFO"
        return "HIGH"

    tests = [
        RuleTest(
            name="Terraform User Agent",
            expected_result=True,
            log={
                "insertId": "-lmjke7dbt7y",
                "logName": "organizations/888888888888/logs/cloudaudit.googleapis.com%2Factivity",
                "p_log_type": "GCP.AuditLog",
                "protoPayload": {
                    "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {
                        "principalEmail": "terraform@platform.iam.gserviceaccount.com",
                        "principalSubject": "serviceAccount:terraform@platform.iam.gserviceaccount.com",
                        "serviceAccountKeyName": "//iam.googleapis.com/projects/platform/serviceAccounts/terraform@platform.iam.gserviceaccount.com/keys/bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb",
                    },
                    "authorizationInfo": [
                        {
                            "granted": True,
                            "permission": "resourcemanager.organizations.setIamPolicy",
                            "resource": "organizations/888888888888",
                            "resourceAttributes": {
                                "name": "organizations/888888888888",
                                "service": "cloudresourcemanager.googleapis.com",
                                "type": "cloudresourcemanager.googleapis.com/Organization",
                            },
                        },
                    ],
                    "methodName": "SetIamPolicy",
                    "request": {
                        "@type": "type.googleapis.com/google.iam.v1.SetIamPolicyRequest",
                        "policy": {
                            "bindings": [
                                {
                                    "members": [
                                        "joey.jojo@example.com",
                                        "serviceAccount:terraform@platform.iam.gserviceaccount.com",
                                    ],
                                    "role": "roles/owner",
                                },
                            ],
                            "etag": "BwXcRFUAtX4=",
                        },
                        "resource": "organizations/888888888888",
                        "updateMask": "bindings,etag,auditConfigs",
                    },
                    "requestMetadata": {
                        "callerIP": "100.100.100.100",
                        "callerSuppliedUserAgent": "Terraform/0.13.2 terraform-provider-google/3.90.1",
                        "destinationAttributes": {},
                        "requestAttributes": {},
                    },
                    "resourceName": "organizations/888888888888",
                    "response": {
                        "@type": "type.googleapis.com/google.iam.v1.Policy",
                        "bindings": [
                            {
                                "members": [
                                    "joey.jojo@example.com",
                                    "serviceAccount:terraform@platform.iam.gserviceaccount.com",
                                ],
                                "role": "roles/owner",
                            },
                        ],
                        "etag": "BwXeRCtKxCw=",
                    },
                    "serviceData": {
                        "@type": "type.googleapis.com/google.iam.v1.logging.AuditData",
                        "policyDelta": {
                            "bindingDeltas": [
                                {"action": "ADD", "member": "user:backdoor@example.com", "role": "roles/owner"},
                            ],
                        },
                    },
                    "serviceName": "cloudresourcemanager.googleapis.com",
                    "status": {},
                },
                "receiveTimestamp": "2022-05-05 14:00:49.450798551",
                "resource": {"labels": {"organization_id": "888888888888"}, "type": "organization"},
                "severity": "NOTICE",
                "timestamp": "2022-05-05 14:00:48.814294000",
            },
        ),
        RuleTest(
            name="Manual Change",
            expected_result=True,
            log={
                "insertId": "-yoga2udnx8s",
                "logName": "organizations/888888888888/logs/cloudaudit.googleapis.com%2Factivity",
                "p_log_type": "GCP.AuditLog",
                "protoPayload": {
                    "@type": "type.googleapis.com/google.cloud.audit.AuditLog",
                    "authenticationInfo": {"principalEmail": "chris@example.com"},
                    "authorizationInfo": [
                        {
                            "granted": True,
                            "permission": "resourcemanager.organizations.setIamPolicy",
                            "resource": "organizations/888888888888",
                            "resourceAttributes": {
                                "name": "organizations/888888888888",
                                "service": "cloudresourcemanager.googleapis.com",
                                "type": "cloudresourcemanager.googleapis.com/Organization",
                            },
                        },
                    ],
                    "methodName": "SetIamPolicy",
                    "request": {
                        "@type": "type.googleapis.com/google.iam.v1.SetIamPolicyRequest",
                        "etag": "BwXYPRNtbqo=",
                        "policy": {
                            "bindings": [
                                {
                                    "members": [
                                        "user:chris@example.com",
                                        "serviceAccount:diana@platform.iam.gserviceaccount.com",
                                    ],
                                    "role": "roles/owner",
                                },
                            ],
                        },
                    },
                    "receiveTimestamp": "2022-02-17T22:52:03.190032712Z",
                    "requestMetadata": {
                        "callerIp": "38.38.38.38",
                        "callerSuppliedUserAgent": "Mozilla/5.0 Chrome/98.0.4758.102",
                        "destinationAttributes": {},
                        "requestAttributes": {},
                    },
                    "resource": {"labels": {"organization_id": "888888888888"}, "type": "organization"},
                    "resourceName": "organizations/888888888888",
                    "response": {
                        "@type": "type.googleapis.com/google.iam.v1.Policy",
                        "bindings": [{"members": ["user:chris@example.com"], "role": "roles/owner"}],
                        "etag": "BwXYPp1Xs/Y=",
                    },
                    "serviceData": {
                        "@type": "type.googleapis.com/google.iam.v1.logging.AuditData",
                        "policyDelta": {
                            "bindingDeltas": [
                                {
                                    "action": "REMOVE",
                                    "member": "serviceAccount:diana@platform.iam.gserviceaccount.com",
                                    "role": "roles/owner",
                                },
                            ],
                        },
                    },
                },
                "serviceName": "cloudresourcemanager.googleapis.com",
                "severity": "NOTICE",
                "status": {},
                "timestamp": "2022-02-17T22:52:02.692083Z",
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.base import key_value_list_to_dict
from pypanther.helpers.crowdstrike_event_streams import cs_alert_context


@panther_managed
class CrowdstrikeNewUserCreated(Rule):
    id = "Crowdstrike.NewUserCreated-prototype"
    display_name = "Crowdstrike New User Created"
    log_types = [LogType.CROWDSTRIKE_EVENT_STREAMS]
    default_severity = Severity.INFO
    create_alert = False
    reports = {"MITRE ATT&CK": ["TA0003:T1136.003"]}
    default_description = "A new Crowdstrike user was created"
    default_runbook = "Confirm the new user is valid."

    def rule(self, event):
        return all([event.deep_get("event", "OperationName") == "createUser", event.deep_get("event", "Success")])

    def title(self, event):
        audit_keys = key_value_list_to_dict(event.deep_get("event", "AuditKeyValues"), "Key", "ValueString")
        actor = event.deep_get("event", "UserId", "UNKNOWN USER")
        target = audit_keys.get("target_name")
        return f"[{actor}] created a new user: [{target}]"

    def alert_context(self, event):
        context = cs_alert_context(event)
        actor = context.get("actor_user", "UNKNWON_ACTOR")
        target = context.get("target_name", "UNKNOWN_TARGET")
        context["actor_target"] = f"{actor}-{target}"
        return context

    tests = [
        RuleTest(
            name="New User Created",
            expected_result=True,
            log={
                "metadata": {
                    "customerIDString": "face_customer_id",
                    "offset": 1238741,
                    "eventType": "AuthActivityAuditEvent",
                    "eventCreationTime": "2024-07-22 15:50:16.923000000",
                    "version": "1.0",
                },
                "event": {
                    "UserId": "bilbo.baggins@hobbiton.co",
                    "UserIp": "1.1.1.1",
                    "OperationName": "createUser",
                    "ServiceName": "CrowdStrike Authentication",
                    "Success": True,
                    "UTCTimestamp": "2024-07-22 15:50:16.923000000",
                    "AuditKeyValues": [{"Key": "target_name", "ValueString": "frodo.baggins@hobbiton.co"}],
                },
            },
        ),
        RuleTest(
            name="Unrelated Event",
            expected_result=False,
            log={
                "event": {
                    "AuditKeyValues": [
                        {"Key": "target_uuid", "ValueString": "e70e5306-4a83-4a9f-9b59-a78c304c438b"},
                        {"Key": "target_cid", "ValueString": "fake_customer_id"},
                        {"Key": "actor_cid", "ValueString": "fake_customer_id"},
                        {"Key": "trace_id", "ValueString": "652fc606f369ef3105925197b34f2c54"},
                        {"Key": "target_name", "ValueString": "peregrin.took@hobbiton.co"},
                        {"Key": "action_target_name", "ValueString": "peregrin.took@hobbiton.co"},
                    ],
                    "OperationName": "userAuthenticate",
                    "ServiceName": "CrowdStrike Authentication",
                    "Success": True,
                    "UTCTimestamp": "2024-07-22 15:50:16.923000000",
                    "UserId": "peregrin.took@hobbiton.co",
                    "UserIp": "1.1.1.1",
                },
                "metadata": {
                    "customerIDString": "fake_customer_id",
                    "eventCreationTime": "2024-07-22 15:50:16.923000000",
                    "eventType": "AuthActivityAuditEvent",
                    "offset": 341329,
                    "version": "1.0",
                },
            },
        ),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_rule_context


@panther_managed
class AWSSecretsManagerBatchRetrieveSecrets(Rule):
    id = "AWS.SecretsManager.BatchRetrieveSecrets-prototype"
    display_name = "AWS Secrets Manager Batch Retrieve Secrets"
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Credential Access", "Stratus Red Team", "Beta"]
    reports = {"MITRE ATT&CK": ["TA0006:T1552"]}
    default_severity = Severity.INFO
    default_description = "An attacker attempted to retrieve a high number of Secrets Manager secrets by batch, through secretsmanager:BatchGetSecretValue (released Novemeber 2023).  An attacker may attempt to retrieve a high number of secrets by batch, to avoid detection and generate fewer calls. Note that the batch size is limited to 20 secrets.\n"
    default_runbook = "https://aws.amazon.com/blogs/security/how-to-use-the-batchgetsecretsvalue-api-to-improve-your-client-side-applications-with-aws-secrets-manager/"
    default_reference = "https://stratus-red-team.cloud/attack-techniques/AWS/aws.credential-access.secretsmanager-batch-retrieve-secrets/"
    threshold = 5
    dedup_period_minutes = 1440
    summary_attributes = ["eventName", "userAgent", "sourceIpAddress", "recipientAccountId", "p_any_aws_arns"]

    def rule(self, event):
        if event.get("eventName") == "BatchGetSecretValue":
            return True
        return False

    def title(self, event):
        user = event.udm("actor_user")
        return f"[{user}] attempted to batch retrieve a large number of secrets from AWS Secrets Manager"

    def alert_context(self, event):
        return aws_rule_context(event)

    tests = [
        RuleTest(
            name="BatchGetSecretValue",
            expected_result=True,
            log={
                "eventSource": "secretsmanager.amazonaws.com",
                "eventName": "BatchGetSecretValue",
                "requestParameters": {"filters": [{"key": "tag-key", "values": ["StratusRedTeam"]}]},
                "responseElements": None,
                "readOnly": True,
                "eventType": "AwsApiCall",
                "managementEvent": True,
                "recipientAccountId": "012345678901",
            },
        ),
    ]



"""
Used to define global helpers and variables for Slack events.
"""


def slack_alert_context(event):
    return {
        "actor-name": event.deep_get("actor", "user", "name", default="<MISSING_NAME>"),
        "actor-email": event.deep_get("actor", "user", "email", default="<MISSING_EMAIL>"),
        "actor-ip": event.deep_get("context", "ip_address", default="<MISSING_IP>"),
        "user-agent": event.deep_get("context", "ua", default="<MISSING_UA>"),
    }



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed


@panther_managed
class SnowflakeStreamExternalShares(Rule):
    id = "Snowflake.Stream.ExternalShares-prototype"
    display_name = "Snowflake External Data Share"
    log_types = [LogType.SNOWFLAKE_DATA_TRANSFER_HISTORY]
    default_severity = Severity.MEDIUM
    reports = {"MITRE ATT&CK": ["TA0010:T1537"]}
    default_description = (
        "Detect when an external share has been initiated from one source cloud to another target cloud."
    )
    default_runbook = "Determine if this occurred as a result of a valid business request."
    tags = ["Snowflake", "[MITRE] Exfiltration", "[MITRE] Transfer Data to Cloud Account"]

    def rule(self, event):
        return all([event.get("SOURCE_CLOUD"), event.get("TARGET_CLOUD"), event.get("BYTES_TRANSFERRED", 0) > 0])

    def title(self, event):
        return f"A data export has been initiated from source cloud {event.get('SOURCE_CLOUD', '<UNKNOWN SOURCE CLOUD>')} in source region {event.get('SOURCE_REGION', '<UNKNOWN SOURCE REGION>')} to target cloud {event.get('TARGET_CLOUD', '<UNKNOWN TARGET CLOUD>')} in target region {event.get('TARGET_REGION', '<UNKNOWN TARGET REGION>')} with transfer type {event.get('TRANSFER_TYPE', '<UNKNOWN TRANSFER TYPE>')} for {event.get('BYTES_TRANSFERRED', '<UNKNOWN VOLUME>')} bytes"

    tests = [
        RuleTest(
            name="Zero byte transfer",
            expected_result=False,
            log={
                "REGION": "US-EAST-2",
                "SOURCE_CLOUD": "AWS",
                "SOURCE_REGION": "US-EAST-2",
                "TARGET_CLOUD": "AWS",
                "TARGET_REGION": "EU-WEST-1",
                "BYTES_TRANSFERRED": 0,
                "TRANSFER_TYPE": "COPY",
            },
        ),
        RuleTest(
            name="Disallowed Share",
            expected_result=True,
            log={
                "REGION": "US-EAST-2",
                "SOURCE_CLOUD": "AWS",
                "SOURCE_REGION": "US-EAST-2",
                "TARGET_CLOUD": "AWS",
                "TARGET_REGION": "EU-WEST-1",
                "BYTES_TRANSFERRED": 61235879,
                "TRANSFER_TYPE": "COPY",
            },
        ),
    ]



from panther_core import PantherEvent

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.aws import aws_cloudtrail_success, aws_regions, aws_rule_context


@panther_managed
class AWSEC2DownloadInstanceUserData(Rule):
    id = "AWS.EC2.DownloadInstanceUserData-prototype"
    display_name = "AWS EC2 Download Instance User Data"
    log_types = [LogType.AWS_CLOUDTRAIL]
    default_severity = Severity.MEDIUM
    reports = {"MITRE ATT&CK": ["TA0007:T1580"]}
    default_description = "An entity has accessed the user data scripts of multiple EC2 instances.\n"
    threshold = 10
    dedup_period_minutes = 1440
    default_reference = "https://hackingthe.cloud/aws/general-knowledge/introduction_user_data/\n"
    default_runbook = "An entity has accessed the user data scripts of multiple EC2 instances. This is often an attempt to find unsecured credentials. Ensure the EC2 instances accessed do not have any sensitive information stored in the user data.\nCloud security scanning tools may trigger false positives.  Add an exclude filter for the scanning tool's service account to prevent false positives.\n"
    summary_attributes = ["userAgent", "sourceIpAddress", "recipientAccountId", "p_any_aws_instance_ids"]
    tags = [
        "AWS CloudTrail",
        "EC2",
        "Discovery",
        "Cloud Infrastructure Discovery",
        "Discovery - Cloud Infrastructure Discovery",
    ]

    def rule(self, event: PantherEvent) -> bool:
        # Panther's Cloud Security Scanning feature triggers false positives
        actor = event.udm("actor_user")
        if (
            any(actor == "PantherAuditRole-" + region for region in aws_regions())
            and event.get("userAgent") == event.get("sourceIPAddress") == "cloudformation.amazonaws.com"
        ):
            return False
        return (
            event.get("eventName") == "DescribeInstanceAttribute"
            and event.deep_get("requestParameters", "attribute") == "userData"
        )

    def title(self, event: PantherEvent) -> str:
        account_id = event.get("recipientAccountId", "UNKNOWN AWS ACCOUNT")
        actor_user = event.udm("actor_user")
        return f"EC2 Instance User Data accessed in bulk by [{actor_user}] in AWS Account [{account_id}]"

    def severity(self, event: PantherEvent) -> str:
        if not aws_cloudtrail_success(event):
            return "LOW"
        return "DEFAULT"

    def alert_context(self, event: PantherEvent) -> dict:
        return aws_rule_context(event)

    tests = [
        RuleTest(
            name="Unsuccessful Attempt",
            expected_result=True,
            log={
                "p_event_time": "2024-12-11 21:49:07.000000",
                "p_log_type": "AWS.CloudTrail",
                "p_parse_time": "2024-12-11 21:55:54.289594",
                "awsRegion": "us-west-2",
                "errorCode": "Client.UnauthorizedOperation",
                "errorMessage": "You are not authorized to perform this operation. User: arn:aws:sts::111122223333:assumed-role/SampleRole/sample-session is not authorized to perform: ec2:DescribeInstanceAttribute on resource: arn:aws:ec2:us-west-2:111122223333:instance/* because no identity-based policy allows the ec2:DescribeInstanceAttribute action. Encoded authorization failure message: 2RCBDBB0PcDpowyBFV7UpJobr-4iGb1MWqjKES--SAO_ca79pxBjKLQQ-sBGm2_m8jjcTAITBlNRa1WCVabq3VCghw9mbj0XZ3-2ZfsVyUrQWiTSUqYh9CUFVnWOmp710QzzKLj-Dy5Aup-bkSCbadoQe6HMevGPiSGeklgwBQc4jLO3oIbPTsujAijnaVr4CUIRBkD4KI8H2ZshEgJOujlpkg47FYqU8bVPNEIv9OwsQ9g4dRMB_cJ7C43kpURWIuiWkFpN8Q84RubskMNB3IfkHGR_Y63Z3LLzOCbPgT4S2zt97PBryCs-lUWSdA2ZCq9dcOtoHQq3Ed-3eB4o7FdZ0cJawa2i6oeUGzzPC2JtBQBcCqExMS8CXENosA0LY9cRUOSo4xc439wzlNpUbMI6K_y1AZWWN0f1QzLW1GkhtlUaLZFvLTwMqTVhClznln2ntmAJ_6iVVlvYco5x6Z4oNn8pMVPk76Iq8yM9o_2-tXmt0st7y_9B83eTSFOTbRnwgHabQXuMyzFKAA3xtuMcbcpB-Ij8c70lkv-eVQeJnfqRd_zbbnsMhxBybfjQyirKjY8bfLg",
                "eventCategory": "Management",
                "eventID": "fa67e0b9-9837-4c84-baa0-78a271821d3e",
                "eventName": "DescribeInstanceAttribute",
                "eventSource": "ec2.amazonaws.com",
                "eventTime": "2024-12-11 21:49:07.000000000",
                "eventType": "AwsApiCall",
                "eventVersion": "1.10",
                "managementEvent": True,
                "readOnly": True,
                "recipientAccountId": "111122223333",
                "requestID": "f2362cc7-4396-4be5-950c-16c36dddde76",
                "requestParameters": {"attribute": "userData", "instanceId": "i-12345678"},
                "sourceIPAddress": "1.2.3.4",
                "tlsDetails": {
                    "cipherSuite": "TLS_AES_128_GCM_SHA256",
                    "clientProvidedHostHeader": "ec2.us-west-2.amazonaws.com",
                    "tlsVersion": "TLSv1.3",
                },
                "userAgent": "sample-user-agent",
                "userIdentity": {
                    "accessKeyId": "SAMPLE_ACCESS_KEY",
                    "accountId": "111122223333",
                    "arn": "arn:aws:sts::111122223333:assumed-role/SampleRole/sample-session",
                    "principalId": "SAMPLE_PRINCIPAL_ID:sample-session",
                    "sessionContext": {
                        "attributes": {"creationDate": "2024-12-11T21:49:04Z", "mfaAuthenticated": "false"},
                        "sessionIssuer": {
                            "accountId": "111122223333",
                            "arn": "arn:aws:iam::111122223333:role/SampleRole",
                            "principalId": "SAMPLE_PRINCIPAL_ID",
                            "type": "Role",
                            "userName": "SampleRole",
                        },
                    },
                    "type": "AssumedRole",
                },
            },
        ),
        RuleTest(
            name="Successful Attempt",
            expected_result=True,
            log={
                "p_event_time": "2024-12-12 21:44:35.000000",
                "p_log_type": "AWS.CloudTrail",
                "p_parse_time": "2024-12-12 21:50:54.711099",
                "awsRegion": "us-west-2",
                "eventCategory": "Management",
                "eventID": "6dbc00d3-1230-492a-ad34-4e3e49aba847",
                "eventName": "DescribeInstanceAttribute",
                "eventSource": "ec2.amazonaws.com",
                "eventTime": "2024-12-12 21:44:35.000000000",
                "eventType": "AwsApiCall",
                "eventVersion": "1.10",
                "managementEvent": True,
                "readOnly": True,
                "recipientAccountId": "111122223333",
                "requestID": "26b6b895-900d-44aa-b286-b8d828c31aed",
                "requestParameters": {"attribute": "userData", "instanceId": "i-1234567890abcdefa"},
                "sourceIPAddress": "1.2.3.4",
                "tlsDetails": {
                    "cipherSuite": "TLS_AES_128_GCM_SHA256",
                    "clientProvidedHostHeader": "ec2.us-west-2.amazonaws.com",
                    "tlsVersion": "TLSv1.3",
                },
                "userAgent": "sample-user-agent",
                "userIdentity": {
                    "accessKeyId": "SAMPLE_ACCESS_KEY",
                    "accountId": "111122223333",
                    "arn": "arn:aws:sts::111122223333:assumed-role/SampleRole/sample-session",
                    "principalId": "SAMPLE_PRINCIPAL_ID:sample-session",
                    "sessionContext": {
                        "attributes": {"creationDate": "2024-12-12T21:43:11Z", "mfaAuthenticated": "false"},
                        "sessionIssuer": {
                            "accountId": "111122223333",
                            "arn": "arn:aws:iam::111122223333:role/SampleRole",
                            "principalId": "SAMPLE_PRINCIPAL_ID",
                            "type": "Role",
                            "userName": "SampleRole",
                        },
                    },
                    "type": "AssumedRole",
                },
            },
        ),
        RuleTest(
            name="Panther Cloud Security Scanning",
            expected_result=False,
            log={
                "p_log_type": "AWS.CloudTrail",
                "awsRegion": "us-west-2",
                "eventCategory": "Management",
                "eventID": "21764f21-382a-4863-8573-908558a839ab",
                "eventName": "DescribeInstanceAttribute",
                "eventSource": "ec2.amazonaws.com",
                "eventTime": "2025-02-09 20:29:17.000000000",
                "eventType": "AwsApiCall",
                "eventVersion": "1.10",
                "managementEvent": True,
                "readOnly": True,
                "recipientAccountId": "123412341234",
                "requestID": "cb8b2a56-d76d-40c2-af25-58c1547d9a48",
                "requestParameters": {"attribute": "userData", "instanceId": "i-1234123412341234"},
                "sourceIPAddress": "cloudformation.amazonaws.com",
                "userAgent": "cloudformation.amazonaws.com",
                "userIdentity": {
                    "accountId": "123412341234",
                    "arn": "arn:aws:sts::123412341234:assumed-role/PantherAuditRole-us-east-1/12341234123412341234",
                    "invokedBy": "cloudformation.amazonaws.com",
                    "principalId": "AROA1234123412341234:12341234123412341234",
                    "sessionContext": {
                        "attributes": {"creationDate": "2025-02-09T20:29:15Z", "mfaAuthenticated": "false"},
                        "sessionIssuer": {
                            "accountId": "123412341234",
                            "arn": "arn:aws:iam::123412341234:role/PantherAuditRole-us-east-1",
                            "principalId": "AROA1234123412341234",
                            "type": "Role",
                            "userName": "PantherAuditRole-us-east-1",
                        },
                    },
                    "type": "AssumedRole",
                },
            },
        ),
    ]
